{ FoxClone
  Copyright (C) 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}
{-------------------------------------------------------------------------------
UNIT - BACKUP

The objects and methods for backup. The backup panel is a child of basepanel. No additional objects
are required so the unit defines the methods required for backup.

V1.0 - 10 Dec 2021
-------------------------------------------------------------------------------}
unit backup;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, CheckLst, StdCtrls, Forms, dialogs, basepanel, drives, settings,
  utilities, il8nstrings, fdialog, dateutils, confform;

type
  TBack = class(TBasePanel)
    private
      Box2Toggle : boolean; //used so that msgbox only appears once when clicking on part checkboxes
      procedure Box1CK(Sender: TObject);
      procedure Box2CK(Sender: TObject);
      procedure PopulateBox2;
      procedure PopulateBox3;
      procedure Box3CK(Sender: TObject);
      procedure PopulateBox4;
      procedure Box4CK(Sender: TObject);
      procedure NextBtnCK(Sender: TObject);
      //testing
      procedure Box2check(Sender: TObject);

    public
      constructor Create(TheOwner: TComponent);
      procedure ClearAll;
  end;

var
  BackTab : TBack;
  BackFileName : string;

implementation
uses restore, clone, partclone; //to avoid circular errors
//not understood, putting partclone in the main uses clause causes a circular error between backup and
//verify
{-------------------------------------------------------------------------------
CONSTRUCTOR TBACK.CREATE
Creates a standard basepanel and adds in the properties, functions & procedures specific to backup.

Used by:

V1.0 - 10 Mar 2021
V1.1 - 03 May 2021 - OnClickCheck added for all boxes, thought OnClick worked for both the text
                     and the checkbox, no it doesn't.
-------------------------------------------------------------------------------}
constructor TBack.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  Box1.BoxType:= bkBox1;
  Box2.BoxType:= bkBox2;
  Box3.BoxType:= bkBox3;
  Box4.BoxType:= bkBox4;
  Box1.OnClick:= @Box1CK;
  Box2.OnClick:= @Box2CK;
  Box2Toggle:= false;
  Box3.OnClick:= @Box3CK;
  Box4.OnClick:= @Box4CK;
  NextBtn.OnClick:= @NextBtnCK;
  Box2.OnClickCheck:=@Box2check;
end; //end constructor TBack.Create

{-------------------------------------------------------------------------------
PROCEDURE TBACK.BOX1CK
Event handler for clicks on box1.

Does basic housekeeping - clearing all selections in drives and partitions and clearing boxes 2 -4.

Only a single drive may be selected.

On a valid click that selects a drive:
   sets that drive as the backup source and sets all partitions on it selected = true
   populates box2 with the partitions on the drive and box3 with the destination drives

If the user clicks on blank space - nothing happens. Did look at using this to clear the existing
selection but was adding to the complexity of the code.

Outstanding issue - slight flicker on selecting the second item, but only sometimes when selecting
sda and then selecting sdb??

Note -  clicking on either the check box or the text triggers this procedure, but if clicking on text it
does not change the state of the checkbox, clicking on the checkbox does change its state. Went round
in circles a few times trying to sort out the logic of this. The initial code on saving the checkbox
state and then resetting it works, but not altogether sure how.

Used by:

V1.0 - 10 Mar 2021
V1.1 - 23 Mar 2021, modified for SystemError
V1.2 - 24 Mar 2021, disable save to file button
V1.3 - 28 May 2021, removed selected = false and modified to use CheckState consistent with all other
                    TCheckListBoxes
-------------------------------------------------------------------------------}
procedure TBack.Box1CK(Sender: TObject);
var
  Source : shortint;
  dindex : shortint;
begin
  //clear all other tabs
  RestoreTab.ClearAll;
  CloneTab.ClearAll;
  //VerifyTab.ClearAll;
  MyDrives.unSetAll(acNone);
  //when clearing other tabs, restore has to unmount any parts mounted by foxclone
  Source:= Box1.ItemIndex;
  if Source < 0 then exit;
  DebugInfo(true,'TBack.Box1CK',Box1.Items[Source]);
  MyDrives.unSetAll(acSource); //button has been clicked so remove all previous drive/part selections
  clearBox(2);       //clear all the other boxes
  clearBox(3);
  clearBox(4);
  NextBtn.Enabled:= false; //reset the save to file button
  Box1.CheckAll(cbUnchecked); //only one can be selected
  Box2Lbl.Enabled:= true;
  Box1.Checked[Source]:= not Box1.CheckState[Source];
  Box1.CheckState[Source]:= Box1.Checked[Source];
  if Box1.Checked[Source] then
     begin
       dindex:= MyDrives.FindName(Box1.Items[Source]);
       if dindex = -1 then SystemError(1,Box1.Items[Source]); //close foxclone on error
       MyDrives[dindex].setAction:= acSource; //set as backup source drive
       PopulateBox2; //populate box2 with partitions in MyDrives[dindex]
       PopulateBox3; //show drives in box3 excluding the backup drive
     end;
end; //end procedure TBack.Box1CK
{-------------------------------------------------------------------------------
PROCEDURE TBACK.BOX2CK
Event handler for clicks on box2.

Partitions cannot be selected until the advanced check box has been enabled on the settings tab.

Shows a info message if an attempt is made to click on a partition without setting 'advanced'.
This is shown only once, which is what the Box2Toggle is used for.

As well as showing the status of the item clicked, sets/unsets the selected flag on the partition in
the selected backup drive (selected in box1).

The user cannot select swap or extended partitions or the EFI partition (if present). This is determined
by UserSelect. UserSelect is a boolean function, it returns false if the partition is EFI, swap or
extended. These partitions are always backed up.

Used by:

V1.0 - 10 Mar 2021
V1.1 - 23 Mar 2021, modified for SystemError and FoxDialog
V1.2 - 11 May 2021, FoxDialog replaced by FoxMsg
V1.2 - 14 May 2021, clicking on a swap/extended/efi part text behaved ok, clicking on the checkbox
                    was able to change state, i.e. enable it. Fix - else clause added to the userselect
                    check.
V1.3 - 25 May 2021, after many tests, settled on using CheckState to monitor state of the checkbox
                    given that clicking on the checkbox flips its state before calling any handler
V1.4 - 28 May 2021, changed to show extended, swap and efi parts as true
-------------------------------------------------------------------------------}
procedure TBack.Box2CK(Sender: TObject);
var
  Source, dindex, pindex : shortint;
begin
  writeln('back box2 clicked on index ',Box2.ItemIndex,' at ',Formatdatetime('hh:nn.zzz ',Now));
  if Box2.ItemIndex < 0 then exit;
  Source:= Box2.ItemIndex;


  Box2.Checked[Source]:= Box2.CheckState[Source]; //in case clicking on checkbox has flipped state
  if Box2.Checked[Source] then writeln('checkbox enabled') else writeln('checkbox NOT enabled');

  DebugInfo(true,'TBack.Box2CK',Box2.Items[Source]);
  if not Defaults.Advanced then //editing of partitions not enabled?
     begin
       if not Box2Toggle then
          begin
            FoxMsg(rPartEditCapStr,[rPartEditStr],mtInformation,[mbOK]);
            Box2Toggle:= true; //so the message only shows once
          end;
       exit;
     end;
  dindex:= MyDrives.getDrive(acSource); //looks for the drive set acSource
  pindex:= MyDrives[dindex].Partitions.FindName(Box2.Items[Source]);
  if pindex = -1 then SystemError(2,Box2.Items[Source]); //close foxclone on error
  if MyDrives[dindex].Partitions[pindex].UserSelect then //check user can enable/disable partitions
     begin
       Box2.Checked[Source]:= Box2.CheckState[Source]; //in case clicking on checkbox has flipped state
       Box2.Checked[Source]:= not Box2.Checked[Source];
       Box2.CheckState[Source]:= Box2.Checked[Source];
       MyDrives[dindex].Partitions[pindex].selected:= Box2.Checked[Source];
     end
  else //user cannot amend state, set it back to initial condition
     begin
       Box2.Checked[Source]:= true;
       Box2.CheckState[Source]:= true;
     end;
end; //end procedure TBack.Box2CK
//testing
{-------------------------------------------------------------------------------
PROCEDURE TBACK.BOX2CHECK
Event handler for clicks on box2.

This is to fix anomalous behaviour when clicking on the checkbox not item text:

     on the first click box2click is called, the message about part selection displayed when ok
     box2click is called again.

     on the 2nd click the box changes state BUT box2click is NOT called, could be a timing issue,
     don't know. BUT the event handler for onclick check = this procedure IS called.

     on the 3rd and subsequent clicks it behaves itself.


As this event handler for onclickcheck seems to be always called, this procedure simply resets the
state of the checkbox based on checkstate - the array previously introduced to try and fix this
problem.

Bit of a kludge if starting with a blank canvas would have been better to simply use the onclickcheck
event handler instead of the onclick handler.

Wonder how many other checkboxes are the same??

Used by:

V1.0 - 21 Sep 2023
-------------------------------------------------------------------------------}
procedure TBack.Box2check(Sender: TObject);
begin
  writeln('onclick check event called');
  if Box2.ItemIndex < 0 then exit;
  //in case clicking on checkbox has flipped state
  Box2.Checked[Box2.ItemIndex]:= Box2.CheckState[Box2.ItemIndex];
end;
{-------------------------------------------------------------------------------
PROCEDURE TBACK.POPULATEBOX2
Populates box2 with the partitions on the drive selected in box1.

All partitions on the drive are set selected = true.

In box2, extended, swap or the EFI partition are shown not checked (as these are not user selectable).


Used by:

V1.0 - 10 Mar 2021
V1.1 - 26 Mar 2021, set scrollbars added.
V1.1 - 25 May 2021, given all the problems with clicks on TCheckListBoxes, amended to use CheckStatus and
                    populate it, so the event handlers can be modified to use it.
V1.2 - 28 May 2021, changed so that extended, swap and efi parts are shown checked
-------------------------------------------------------------------------------}
procedure TBack.PopulateBox2;
var
  i, dindex : byte;
begin
  dindex:= MyDrives.getDrive(acSource); //get the drive selected for backup
  Box2.enabled:= true;
  for i:= 0 to MyDrives[dindex].TotalParts - 1 do with MyDrives[dindex].partitions[i] do
      begin
        Box2.Items.add(ListStr);
        selected:= true; //mark all the partition selected
        Box2.Checked[i]:= true; //show all parts in the checkbox true
      end;
  //setup and populate CheckState
  setLength(Box2.CheckState,Box2.Count);
  for i:= 0 to Box2.Count - 1 do Box2.CheckState[i]:= Box2.Checked[i];
  setScrollBarTBox(Box2);
end; //end procedure TBack.populateBox2
{-------------------------------------------------------------------------------
PROCEDURE TBACK.POPULATEBOX3
Populates box3 with the drives available as the backup destination.

Excludes the source drive or a blank unformatted drive.

Used by:

V1.0 - 10 Mar 2021
V1.1 - 26 Mar 2021, scroll bars added
V1.2 - 28 May 2021, modified to setup checkstate
-------------------------------------------------------------------------------}
procedure TBack.PopulateBox3;
var
  i : byte;
begin
  Box3.Enabled:= true;
  Box3Lbl.Enabled:= true;
  for i:= 0 to MyDrives.Count - 1 do with MyDrives[i] do
      if not ((setAction = acSource) or isBlank) then Box3.Items.add(ListStr);
  //setup CheckState
  setLength(Box3.CheckState,Box3.Count);
  for i:= 0 to Box3.Count - 1 do Box3.CheckState[i]:= false; //default, nothing selected
  setScrollBarTBox(Box3);
end; //end procedure TBack.PopulateBox3
{-------------------------------------------------------------------------------
PROCEDURE TBACK.BOX3CK
Event handler for clicks on box3.

After housekeeping, sets the drive selected as acTarget, then runs PopulateBox4 to display the
partitions on the drive selected as the destination for a backup in box4.

Used by:

V1.0 - 23 Mar 2021
V1.1 - 24 Mar 2021 disable the save to file button
V1.2 - 28 May 2021, removed selected = false, modified to use checkstate
-------------------------------------------------------------------------------}
procedure TBack.Box3CK(Sender: TObject);
var
  Source : shortint;
  dIndex : shortint;
begin
  Source:= Box3.ItemIndex;
  if Source < 0 then exit;
  DebugInfo(true,'TBack.Box3CK',Box3.Items[Source]);
  MyDrives.unSetAll(acTarget); //reset the drive status
  clearBox(4);
  NextBtn.Enabled:= false;
  Box3.CheckAll(cbUnchecked); //only one can be selected
  Box3.Checked[Source]:= not Box3.CheckState[Source];
  Box3.CheckState[Source]:= Box3.Checked[Source];
  if Box3.Checked[Source] then
     begin
       dIndex:= MyDrives.FindName(Box3.Items[Source]);
       if dIndex = -1 then SystemError(5,Box3.Items[Source]); //close foxclone on error
       MyDrives[dIndex].setAction:= acTarget; //set as target
       PopulateBox4; //show partitions in box4
     end;
end; //end TBack.Box3CK
{-------------------------------------------------------------------------------
PROCEDURE TBACK.POPULATEBOX4
Populates box4 with the partitions available as the backup destination.

Does a bit of basic housekeeping first, e.g. clearing box 4 before searching for the drive set as
acTarget and adding the partitions on it.

Used by:

V1.0 - 24 Mar 2021
V1.1 - 26 Mar 2021, scroll bars added
V1.2 - 28 May 2021, modified to setup checkstate
V1.3 - 29 Jan 2022, not checking if there are any partitions on the target drive. Error if the user
                    selects a drive with a part table but no partitions.
-------------------------------------------------------------------------------}
procedure TBack.PopulateBox4;
var
  dindex, i : shortint;
begin
  clearBox(4);
  NextBtn.Enabled:= false;
  Box4Lbl.enabled:= true;
  Box4.enabled:= true;
  dindex:= MyDrives.getDrive(acTarget);
  //cant add a swap part and extended part or an efi part
  if MyDrives[dindex].TotalParts > 0 then with MyDrives[dindex] do
     begin
       for i:= 0 to TotalParts - 1 do
       if Partitions[i].UserSelect then Box4.Items.add(Partitions[i].ListStr);
       //setup CheckState
       setLength(Box4.CheckState,Box4.Count);
       for i:= 0 to Box4.Count - 1 do Box4.CheckState[i]:= false; //default, nothing selected
       setScrollBarTBox(Box4);
     end;
end; //end procedure TBack.PopulateBox4
{-------------------------------------------------------------------------------
PROCEDURE TBACK.BOX4CK
Event handler for clicks on box4.

After housekeeping looks for the drive set acTarget and then finds the partition selected.
EFI, swap or extended partitions cannot be selected. For other partitions, sets the partition selected
in the checklistbox to selected = true and checks if enough space on it. If this test is passed, then
the 'save to file' button is enabled.

Used by:

V1.0 - 24 Mar 2021
V1.1 - 28 May 2021, removed selected = false, modified to use checkstate
V1.2 - 22 Oct 2021, Missed resetting selected on acTarget partition when not enough space, fixed.
-------------------------------------------------------------------------------}
procedure TBack.Box4CK(Sender: TObject);
var
  Source : shortint;
  i, dindex, pindex : shortint;
begin
  NextBtn.Enabled:= false; //reset
  Source:= Box4.ItemIndex;
  DebugInfo(true,'TBack.Box4CK itemindex ',inttostr(Source));
  if Source < 0 then exit;
  DebugInfo(true,'TBack.Box4CK',Box4.Items[Source]);
  Box4.CheckAll(cbUnchecked); //only one can be selected
  Box4.Checked[Source]:= not Box4.CheckState[Source];
  Box4.CheckState[Source]:= Box4.Checked[Source];
  if Box4.Checked[Source] then //find out if enough space available on destination
     begin
       dindex:= MyDrives.getDrive(acTarget); //find the destination drive
       if dIndex = -1 then SystemError(6,'TBack.Box4CK'); //close foxclone on error
       //deselect all partitions
       with MyDrives[dindex] do for i:= 0 to TotalParts - 1 do Partitions[i].selected:= false;
       //find the partition selected
       pindex:= MyDrives[dindex].Partitions.FindName(Box4.Items[Source]);
       if pindex = -1 then SystemError(11,Box4.Items[Source]);
       //need to check if extended, swap or EFI - cannot select
       if not MyDrives[dindex].Partitions[pindex].UserSelect then
          begin
            Box4.Checked[Source]:= false;
            Box4.CheckState[Source]:= false;
            exit;
          end;
       MyDrives[dindex].Partitions[pindex].selected:= true; //set the partition as the destination
       if not MyDrives.CheckSpace then //is there enough space, true = okay?
          begin
            Box4.Checked[Source]:= false;
            Box4.CheckState[Source]:= false;
            MyDrives[dindex].Partitions[pindex].selected:= false;
          end
       else NextBtn.Enabled:= true; //there is enough space - enable save to file
     end
end; //end TBack.Box4CK
{-------------------------------------------------------------------------------
PROCEDURE TBACK.CLEARALL
A wrapper to clear all boxes in the backup tab.

Used by: TBasePanel.CreateNew

V1.0 - 30 Mar 2021
-------------------------------------------------------------------------------}
procedure TBack.ClearAll;
begin
  with Self do
      begin
        ClearBox(1);
        ClearBox(2);
        ClearBox(3);
        ClearBox(4);
      end;
end; //end procedure TBack.ClearAll
{-------------------------------------------------------------------------------
PROCEDURE TBACK.NEXTBTNCK
Event handler for clicks on the next button. The main function is to get the backup filename from the
user.

Assuming the user has not cancelled, the next step in a backup is triggered by calling confirm and
then running runFoxClone.

Used by:

V1.0 - 02 Sep 2021
V1.1 - 05 Oct 2021, modded for partclone
V1.2 - 20 Oct 2021, savedialog.filter added to stop display of other files in the save dialog
V1.3 - 25 Jan 2022, if foxclone has mounted the partition to save the backup in, it umounts it at
                    the end of the procedure. If user clicks to add a note = error, nowhere to save
                    the note file. Removed, umounting now done in the method for closing the prog form.
V1.4 - 03 Sep 2024, check to add .backup to the filename if the user has entered a filename with a
                    period/full stop. Now amended to removing SaveDialog.DefaultExt and manually adding
                    the .backup extension to the filename added by the user.
-------------------------------------------------------------------------------}
procedure TBack.NextBtnCK(Sender: TObject);
const
  backextn : string = '.backup';
var
  SaveDialog : TSaveDialog;
  Today : string; //today's date in format YYYYMMDD
  i : integer;
  dIndex, pIndex : shortint;
  mountpoint : string;
begin
  //get the backup file name.
  //get today's date as starting point for backup filename, format YYYYMMDD

  Today:= FormatDateTime('yyyymmdd',now);
  //writeln('today is ',Today);
  DebugInfo(true,'TBack.NextBtnCK');

  //now where is mount point drive with acTarget set and selected = true on the partition
  dindex:= MyDrives.getDrive(acTarget); //find the destination drive
  if dIndex = -1 then SystemError(6,'TBack.NextBtnCK'); //close foxclone on error
  pIndex:= -1;
  with MyDrives[dIndex] do for i:= 0 to TotalParts - 1 do if Partitions[i].selected then pIndex:= i;
  if pIndex = -1 then SystemError(11,'TBack.NextBtnCK');
  //writeln('dIndex =',dindex);
  //writeln('pindex=',pindex);
  mountpoint:= MyDrives[dIndex].Partitions[pIndex].Mount(true); //mount the target partition
  //writeln('mountpoint =',mountpoint);

  //open save dialog for file name and location
  SaveDialog:= TSaveDialog.Create(Self);
  SaveDialog.Filename:= today;
  SaveDialog.InitialDir:= MountPoint;
  //SaveDialog.DefaultExt:= '.backup';  //this automatically adds .backup to the filename.
  //except it doesnt if the user has used a period in the filename they entered, now manually add
  SaveDialog.Filter:= 'Backup file|*.backup';
  if SaveDialog.Execute then
     begin
       //the user may have selected a different drive/partition from the left hand panel on the
       //save dialog. This must be trapped.
       if pos(mountpoint,SaveDialog.Filename) > 0 then
          begin
            BackFileName:= SaveDialog.Filename + backextn;
            //if pos(backextn,BackFileName) = 0 then BackFileName:= BackFileName + backextn;
            DebugInfo(true,'TBack.NextBtnCK','BackFile',BackFileName);
            writeln('TBack.NextBtnCK',' backfname=',BackFileName);
            if Confirm(acBackup,BackFileName) then runFoxClone(acBackup,BackFileName);
          end
       else foxmsg(rSaveTitleStr,[rSaveMsgStr1,rSaveMsgStr2],mtError,[mbOK]);
     end;
  //if foxclone mounted the destination then need to unmount it
  //if pos(MountFolder,MountPoint) > 0 then MyDrives[dIndex].Partitions[pIndex].Umount;
  SaveDialog.Free;
end; //end procedure TBack.NextBtnCK

end.

