{ FoxClone
  Copyright (C) 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}
{-------------------------------------------------------------------------------
UNIT - CLONE

The objects and methods for clone - both clone drive to drive and from backup to drive.
The clone panel is a child of the restore panel. No additional objects
are required so the unit defines the methods required for clone.

Note - CloneComboBox is defined in basepanel.

V1.0 - 01 Jun 2021
-------------------------------------------------------------------------------}
unit clone;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, CheckLst, StdCtrls, Forms, Controls, dialogs, strutils, DateUtils, Graphics, restore,
  basepanel, backup, GTKlabel, drives, partitions, settings, utilities, il8nstrings, fdialog,
  confform;

type
  TClone = class(TRestore)
    private
      procedure CloneType(Sender : TObject);
      procedure Box1CK(Sender: TObject);
      procedure PopulateBox2;
      procedure Box2CK(Sender : TObject);
      procedure PopulateBox3;
      procedure Box3CK(Sender: TObject);
      procedure Box4CK(Sender : TObject);
      procedure Box4CheckCK(Sender: TObject);
      procedure NextBtnCK(Sender: TObject);
    public
      constructor Create(TheOwner: TComponent);
      procedure ClearAll;
  end;

var
  CloneTab : TClone;

implementation
uses partclone;
{-------------------------------------------------------------------------------
CONSTRUCTOR TCLONE.CREATE
Creates the clone panel and adds in the properties, functions & procedures specific to clone.

clonetab is an amended copy of restoretab.

Used by:

V1.0 - 01 Jun 2021
V1.1 - 02 Jun 2021, replaced Box3 with LBox3, took a while to track down all the references to Box3
                    to remove the SIGSEGV faults. It didn't like freeandnil(Box3) replaced with
                    Box3.free which was okay. Sorted, it was the clearall procedure calling clearbox(3).
V1.2 - 28 Sep 2021, back to Box3 as a checklistbox
V1.3 - 28 Oct 2021, no changes but not showing hints on box3, adding in from trestore made no difference
V1.4 - 27 Nov 2021, box3 was showing on creation, set visible false
V1.5 - 11 Jan 2021, nextbtn anchor changed to box 1.
V1.6 - 12 Jan 2022, onmouse leave event added to box4.
-------------------------------------------------------------------------------}
constructor TClone.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  TitleLbl.setCaption(rCloneTabStr);
  ActionImg.Picture.LoadFromFile(CloneImgFile);
  ActionImg.BorderSpacing.left:= wWidth - CloneImgWidth;
  ActionImg.BorderSpacing.top:= phBorder * 2;
  Box1.BoxType:= clBox1;
  Box2.BoxType:= clBox2;
  Box3.BoxType:= clBox3; //don't think it is used anywhere
  CloneComboBox:= TComboBox.Create(Self);
  with CloneComboBox do begin
       Parent:= self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= self;
       BorderSpacing.top:= 20;
       AnchorSide[akLeft].Side:= asrRight;
       AnchorSide[akLeft].Control:= TitleLbl;
       BorderSpacing.left:= 20;
       Style:= csDropDownList;
       Items.Add(rCloneComboStr1);
       Items.Add(rCloneComboStr2);
       if GetTextWidth(rCloneComboStr1,font) > GetTextWidth(rCloneComboStr2,font) then
          Width:= GetTextWidth(rCloneComboStr1,font)
       else Width:= GetTextWidth(rCloneComboStr2,font);
       Width:= Width + 30;
       ItemIndex:= 0; //set default as clone drive to drive
       ShowHint:= true;
       Hint:= rCloneComboHint;
       OnChange:= @CloneType;
  end;
  Box2Lbl.setCaption(rCloneBox2Str);
  Box2.OnMouseMove:= @DriveHint;
  Box3Lbl.Visible:= false;
  Box3.Visible:= false;
  RDateLbl.AnchorSide[akTop].Control:= Box3;
  NoteBtn.AnchorSide[akTop].Control:= Box3;
  Box4Lbl.Visible:= false;
  Box4Lbl.setCaption(rCloneBox4Str);
  freeandnil(LBox4);
  Box4:= TFoxCheckBox.Create(Self);
  with Box4 do begin
       BoxType:= clBox4;
       Parent:= self;
       Style:= lbOwnerDrawFixed;//lbOwnerDrawFixed; what is the difference - the item height
       OnDrawItem:= @BoldOS;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= Box4Lbl;
       BorderSpacing.top:= phBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Box2;
       BorderSpacing.left:= 0;
       Width:= Box1.Width;
       Height:= Box1.Height;
       ItemHeight:= BoxItemHeight; //added in v1.7
       ExtendedSelect:= false; //stops shift plus click
       MultiSelect:= false;//stops ctrl plus click
       ShowHint:= true;
       OnMouseMove:= @DriveHint; //to get custom hints
       OnMouseLeave:= @BoxMouseLeave;
       Enabled:= false;
       Visible:= false
  end;
  with NextBtn do begin //when box4 not visible height must be zero
       AnchorSide[akTop].Control:= Box1;
       BorderSpacing.top:= phBorder * 5 + Box1.Height + Box1Lbl.Height;
       Caption:= rCloneTabStr;
  end;
  freeandnil(RExisting); //not used in clone
  //Event handlers
  Box1.OnClick:= @Box1CK;
  Box2.OnClick:= @Box2CK;
  Box3.OnClick:= @Box3CK; //new on switch from listbox to checkbox
  Box4.OnClick:= @Box4CK;
  Box4.OnClickCheck:= @Box4CheckCK;
  NextBtn.OnClick:= @NextBtnCK;
end; //end constructor TClone.Create
{-------------------------------------------------------------------------------
PROCEDURE TCLONE.CLONETYPE
Event handler for the comboxbox selecting either clone drive to drive or clone backup file to drive.

Used by:

V1.0 - 01 Jun 2021
V1.1 - 03 Jun 2021, BoxTypes added for box2 and handlers for hintstrings added
V1.2 - 28 Sep 2021, Box3.Visible:= true/false was LBox3
-------------------------------------------------------------------------------}
procedure TClone.CloneType(Sender : TObject);
begin
  //Reset everything
  BackTab.ClearAll;
  RestoreTab.ClearAll;
  MyDrives.unSetAll(acNone);
  Box1.Clear;
  ClearAll;
  ShowDateAndNote(false);
  NextBtn.Enabled:= false;
  case CloneComboBox.ItemIndex of
       0 : begin //Drive to drive clone
             DebugInfo(true,'TClone.CloneType','Drive to Drive');
             ActionImg.Picture.LoadFromFile(CloneImgFile);
             ActionImg.BorderSpacing.left:= wWidth - CloneImgWidth;
             ActionImg.BorderSpacing.top:= phBorder * 2;
             Box2Lbl.setCaption(rCloneBox2Str);
             Box3Lbl.Visible:= false;
             Box3.Visible:= false;
             Box4Lbl.Visible:= false;
             Box4.Visible:= false;
             Box1.BoxType:= clBox1;
             Box2.BoxType:= clBox2;
             Box2.OnMouseMove:= @DriveHint;
             PopulateBox1;
           end;
       1 : begin //File to drive clone
             DebugInfo(true,'TClone.CloneType','Backup file to Drive');
             ActionImg.Picture.LoadFromFile(CloneFileImgFile);
             ActionImg.BorderSpacing.left:= wWidth - CloneFileImgWidth;
             ActionImg.BorderSpacing.top:= phBorder;
             Box2Lbl.setCaption(rRestBox2Str);
             Box3Lbl.Visible:= true;
             Box3.Visible:= true;
             Box4Lbl.Visible:= true;
             Box4.Visible:= true;
             Box1.BoxType:= clBox1File;
             Box2.BoxType:= clBox2File;
             Box2.OnMouseMove:= @PartHint;
             PopulateBox1;
           end;
  end;
end; //end procedure TClone.CloneType
{-------------------------------------------------------------------------------
PROCEDURE TCLONE.CLEARALL
A wrapper to clear all boxes in the clone tab.

A copy of TBack.ClearAll, as TClone is based on TRestore can't use inherited ClearAll because that has
a listbox.

Used by:

V1.0 - 01 Jun 2021
V1.1 - 02 Jun 2021, Box3 replaced with listbox, LBox3.
V1.2 - 20 Jun 2021, was missing clear date and note
V1.3 - 24 Nov 2021, was not clearing box3 (back to a checklistbox and not disabling the ok button.
-------------------------------------------------------------------------------}
procedure TClone.ClearAll;
begin
  with Self do begin
       ClearBox(1);
       ClearBox(2);
       ClearBox(3);
       ClearBox(4);
       ShowDateAndNote(false);
       NextBtn.Enabled:= false;
  end;
end; //end procedure TClone.ClearAll
{-------------------------------------------------------------------------------
PROCEDURE TCLONE.BOX1CK
Event handler for clicks on box1.

Modified version of TRestore.Box1CK.

Only a single drive may be selected.

On a valid click that selects a drive:
   sets that drive as the source for the backup file or the drive to be cloned
   populates box2 with either the target drives or the partitions on the drive

If the user clicks on blank space - nothing happens.

Used by:

V1.0 - 03 Jun 2021
-------------------------------------------------------------------------------}
procedure TClone.Box1CK(Sender: TObject);
var
  Source : shortint;
  dindex : shortint;
begin
  //clear all other tabs
  BackTab.ClearAll;
  RestoreTab.ClearAll;
  MyDrives.unSetAll(acNone);
  Source:= Box1.ItemIndex;
  if Source < 0 then exit;
  DebugInfo(true,'TClone.Box1CK',Box1.Items[Source]);
  clearBox(2);  //clear all the other boxes
  clearBox(30); //listbox for clone
  clearBox(4);
  ShowDateAndNote(false); //make backup date and note button invisible
  NextBtn.Enabled:= false; //reset the save to file button
  Box1.CheckAll(cbUnchecked); //only one can be selected
  Box2Lbl.Enabled:= true;
  Box1.Checked[Source]:= not Box1.CheckState[Source];
  Box1.CheckState[Source]:= Box1.Checked[Source];
  if Box1.Checked[Source] then
     begin
       dindex:= MyDrives.FindName(Box1.Items[Source]);
       if dindex = -1 then SystemError(19,Box1.Items[Source]); //close foxclone on error
       MyDrives[dindex].setAction:= acSource; //set as the drive to be cloned or containing the backup
       PopulateBox2; //populate box2 with either drives or partitions
     end;
end; //end procedure TClone.Box1CK
{-------------------------------------------------------------------------------
PROCEDURE TCLONE.POPULATEBOX2
Populates box2 with the target drives or the partitions on the drive selected in box1.

Used by:

V1.0 - 03 Jun 2021
-------------------------------------------------------------------------------}
procedure TClone.PopulateBox2;
var
  i, dindex : byte;
begin
  Box2.enabled:= true;
  if CloneComboBox.ItemIndex = 0 then //clone drive to drive
     begin //funny the for if is a single statement but without begin/end get compile error
       for i:= 0 to MyDrives.Count - 1 do
           if not ((MyDrives[i].isBoot) or (MyDrives[i].setAction = acSource))
           then Box2.Items.add(MyDrives[i].ListStr)
     end
  else //clone from backup file
     begin
       dindex:= MyDrives.getDrive(acSource); //get the drive selected containing the backup
       for i:= 0 to MyDrives[dindex].TotalParts - 1 do
           with MyDrives[dindex].partitions[i] do if UserSelect then Box2.Items.add(ListStr);
     end;
  //setup CheckState
  setLength(Box2.CheckState,Box2.Count);
  for i:= 0 to Box2.Count - 1 do Box2.CheckState[i]:= Box2.Checked[i];
  setScrollBarTBox(Box2);
end; //end procedure TRestore.PopulateBox2
{-------------------------------------------------------------------------------
PROCEDURE TCLONE.BOX2CK
onclick handler for clone box2.

Has to handle either a box populated with drives for clone drive to drive, or a box populated with
partitions for clone from backup to drive.

When drive to drive, checks if enough space on the destination drive. If okay enables the next button.

When backup file to drive, prompts for the backup file, parses it and populates listbox 3 with the
partitions in the backup. Populates box4 with the target drives - excludes the boot drive and the
drive the backup is on.

Used by:

V1.0 - 07 Jun 2021
V1.1 - 28 Sep 2021, changing from a listbox to tchecklistbox for box3, clearbox(30) changed to
                    clearbox(3)
V1.2 - 29 Dec 2021, msg strings had not been changed since removing blockadjust and message type changed
                    to error when not enough space. Mixing up drive size = blocks with sectors = sectors
V1.3 - 05 Jan 2022, changed from displaying sizes in blocks to MB when the clone will not fit
V1.4 - 26 Jan 2022, now checks that the source partition is not an unknown filesystem, same change as
                    done on restore box2 click.
-------------------------------------------------------------------------------}
procedure TClone.Box2CK(Sender : TObject);
const
  SdriveStr = '[Sdrive]';
  TdriveStr = '[Tdrive]';
  SBlockStr = '[sblocks]';
  GPTstr = 'gpt';
  UnknownStr = 'unknown';
var
  Source : shortint;
  dsIndex, dtIndex, pIndex : shortint;
  Msg1, Msg2, Msg3, Msg4 : string; //used in the message if the clone won't fit
  i : byte;
  PartEnd : int64;
  LastPart : int64 = 0;
begin
  ClearBox(3);
  ClearBox(4);
  NextBtn.Enabled:= false; //reset
  ShowDateAndNote(false); //make backup date and note button invisible
  Source:= Box2.ItemIndex;
  if Source < 0 then exit;
  DebugInfo(true,'TClone.Box2CK',Box2.Items[Source]);
  Box2.CheckAll(cbUnchecked); //only one can be selected
  Box2.Checked[Source]:= not Box2.CheckState[Source];
  Box2.CheckState[Source]:= Box2.Checked[Source];
  if (CloneComboBox.ItemIndex = 0) and (Box2.Checked[Source]) then //clone drive to drive
     begin //check drive space
       dsIndex:= MyDrives.getDrive(acSource); //find source drive
       //find where the last partition ends
       with MyDrives[dsIndex] do for i:= 0 to MyDrives[dsIndex].TotalParts -1 do
           begin
             with Partitions[i] do PartEnd:= StartSector + BlockToSector(Size,SectorSize);
             if PartEnd > LastPart then LastPart:= PartEnd;
           end;
       //if a gpt drive add on space for the secondary partition table at the end of the drive
       if MyDrives[dsIndex].pTable = GPTstr then LastPart+= 34;
       //writeln('last part=',lastpart);
       dtIndex:= MyDrives.FindName(Box2.Items[Source]); //get the destination drive
       MyDrives[dtIndex].setAction:= acTarget; //set the target drive
       if SectorToBlock(LastPart,MyDrives[dsIndex].DriveSector) > MyDrives[dtIndex].DriveSize then
          begin //clone will not fit on target
            Msg1:= replacestr(rNoCloneDriveStr,SdriveStr,MyDrives[dsIndex].MountPt);
            Msg1:= replacestr(Msg1,TdriveStr,MyDrives[dtIndex].MountPt);
            Msg2:= replacestr(rNoCloneSourceStr,SdriveStr,MyDrives[dsIndex].MountPt);
            Msg2:= replacestr(Msg2,SBlockStr,FormatMB(SectorToBlock(LastPart,MyDrives[dsIndex].DriveSector)));
            Msg3:= rNoCloneTargetStr + FormatMB(MyDrives[dtIndex].DriveSize);
            Msg4:= replacestr(rNoCloneMsgStr,SdriveStr,MyDrives[dsIndex].MountPt);
            FoxMsg(rNoCloneTitleStr,[Msg1,Msg2,Msg3,Msg4],mtError,[mbOK]);
            Box2.Checked[Source]:= false;
            Box2.CheckState[Source]:= false;
          end
       else NextBtn.Enabled:= true;
       exit;
     end;
  //clone backup file to drive - below an amended version of TRestore.Box2CK
  if Box2.Checked[Source] then
     begin
       dsIndex:= MyDrives.getDrive(acSource); //find the drive containing the backup
       if dsIndex = -1 then SystemError(20,'TClone.Box2CK'); //close foxclone on error
       //get the partition
       pindex:= MyDrives[dsIndex].Partitions.FindName(Box2.Items[Source]);
       if pindex = -1 then SystemError(15,Box2.Items[Source]);
       //check the filesystem is not unknown, if it is do nothing
       if MyDrives[dsIndex].Partitions[pIndex].FileSystem = UnknownStr then
         begin
           Box2.Checked[Source]:= false;
           Box2.CheckState[Source]:= false;
           exit;
         end;
       //check the partition is not extended, swap or EFI
       //not sure this is needed, box should not be displaying these partitions
       if not MyDrives[dsIndex].Partitions[pIndex].UserSelect then
         begin
           Box2.Checked[Source]:= false;
           Box2.CheckState[Source]:= false;
           exit;
         end;
       //get and check for restore filename, populates RestFname
       if not GetFile(dsIndex,pIndex) then
          begin
            Box2.Checked[Source]:= false;
            Box2.CheckState[Source]:= false;
            exit;
          end;
       //parse the backup file into RDrive
       if not ReadBackupFile then
          begin
            Box2.Checked[Source]:= false;
            Box2.CheckState[Source]:= false;
            exit;
          end;
       PopulateBox3;
       ShowDateAndNote(true);
       //PopulateBox4;
       for i:= 0 to MyDrives.Count - 1 do with MyDrives[i] do
           if not ((isBoot) or (setAction = acSource)) then Box4.Items.add(ListStr);
       //setup CheckState
       setLength(Box4.CheckState,Box4.Count);
       for i:= 0 to Box4.Count - 1 do Box4.CheckState[i]:= false; //default, nothing selected
       //set scrollbars
       setScrollBarTBox(Box4);
       Box4.Enabled:= true;
       Box4Lbl.Enabled:= true;
     end;
end; //end procedure TClone.Box2CK
{-------------------------------------------------------------------------------
PROCEDURE TCLONE.POPULATEBOX3
Populates listbox3 with the partitions in the selected backup file.

Used by:

V1.0 - 05 Jun 2021
V1.1 - 14 Sep 2021, modded to use chopstr (in resizefilename) to truncate the backup filename
V1.2 - 28 Sep 2021, changed from listbox to tfoxchecklistbox, checkstate and checked set. Name change
                    from PopulateLBox3 to PopulateBox3
V1.3 - 23 Nov 2021, 'partition table' added to box3 for consistency with restore box3, that way can
                    use the same methods for clone from file (no index out of bounds).
V1.4 - 19 Feb 2022, debug info added
-------------------------------------------------------------------------------}
procedure TClone.PopulateBox3;
var
  i : byte;
begin
  DebugInfo(true,'TClone.PopulateBox3');
  setLength(Box3.CheckState,RDrive.TotalParts + 1); //need +1 to include the part table
  //add partition table to the checklist box
  Box3.Items.add(rPartTableStr);
  Box3.CheckState[0]:= true;
  Box3.Checked[0]:= true;
  //set length of the checkstate array
  for i:= 0 to RDrive.TotalParts - 1 do
      begin
        Box3.Items.add(RDrive.Partitions[i].ListStr);
        Box3.CheckState[i+1]:= RDrive.Partitions[i].selected; //need +1 because part table is item 0
        Box3.Checked[i+1]:= Box3.CheckState[i];
      end;
  setScrollBarTBox(Box3);
  Box3.Enabled:= true;
  Box3Lbl.Enabled:= true;
  ReSizeFilename;
end; //end procedure TClone.PopulateBox3
{-------------------------------------------------------------------------------
PROCEDURE TCLONE.BOX4CK
Event handler for box4 selecting the drive to clone to. Checks the target drive to see if there is
enough space, reports an error if not. If everything okay, sets the target drive acTarget and enables
the next button.

Used by:

V1.0 - 05 Jun 2021
V1.1 - 29 Dec 2021, fixed errors in the message and was comparing size = blocks with sectors = sectors
                    as a result of removing blockadjust.
V1.2 - 05 Jan 2022, changed from displaying sizes in blocks to MB when clone will not fit
-------------------------------------------------------------------------------}
procedure TClone.Box4CK(Sender : TObject);
const
  TdriveStr = '[Tdrive]';
  GPTstr = 'gpt';
var
  Source : shortint;
  dindex : shortint;
  PartEnd : int64;
  LastPart : int64 = 0;
  i : byte;
  Msg1, Msg2, Msg3 : string;
begin
  //writeln('box4 clicked...');
  Source:= Box4.ItemIndex;
  if Source < 0 then exit;
  DebugInfo(true,'TClone.Box4CK',Box4.Items[Source]);
  Box4.CheckAll(cbUnchecked); //only one can be selected
  Box4.Checked[Source]:= not Box4.CheckState[Source];
  Box4.CheckState[Source]:= Box4.Checked[Source];
  if Box4.Checked[Source] then
     begin
         dIndex:= MyDrives.FindName(Box4.Items[Source]);
         if dIndex = -1 then SystemError(22,Box4.Items[Source]); //close foxclone on error
         //get end of last part for the backup so can check if the backup will fit on the target
         with RDrive do for i:= 0 to TotalParts - 1 do
             begin
               with Partitions[i] do PartEnd:= StartSector + BlockToSector(Size,SectorSize);
               if PartEnd > LastPart then LastPart:= PartEnd;
             end;
         if RDrive.pTable = GPTstr then LastPart+= 34; //allow for backup partition table.
         //writeln('last part=',lastpart);
         if SectorToBlock(LastPart,RDrive.DriveSector) > MyDrives[dIndex].DriveSize then //backup won't fit on the drive
            begin
              Msg1:= replacestr(rNoCloneBackupStr,TdriveStr,MyDrives[dIndex].MountPt);
              Msg2:= rNoCloneBackSizeStr + FormatMB(SectorToBlock(LastPart,RDrive.DriveSector));
              Msg3:= rNoCloneTargetStr + FormatMB(MyDrives[dIndex].DriveSize);
              FoxMsg(rNoCloneTitleStr,[Msg1,Msg2,Msg3,rNoCloneMsg2Str],mtError,[mbOK]);
              Box4.Checked[Source]:= false;
              Box4.CheckState[Source]:= false;
              exit;
            end;
         MyDrives[dIndex].setAction:= acTarget; //set the target for the clone
         NextBtn.Enabled:= true;
     end;
end; //end procedure TClone.Box4CK
{-------------------------------------------------------------------------------
PROCEDURE TCLONE.BOX4CHECKCK
no idea why, but the first click on clone box4 does not trigger the onclick handler?????

But clicking on the checkbox will fire the onclickcheck handler, hence the need for this procedure to
reset the checkbox to its previous state.

Part of the ongoing issues associated with TCheckListBoxes and clicking on them.

Used by:

V1.0 - 07 June 2021
-------------------------------------------------------------------------------}
procedure TClone.Box4CheckCK(Sender: TObject);
begin
  if Box4.ItemIndex >= 0 then with Box4 do Checked[ItemIndex]:= CheckState[ItemIndex];
end; //end procedure TClone.Box4CheckCK
{-------------------------------------------------------------------------------
PROCEDURE TCLONE.NEXTBTNCK
Event handler for clicking on the clone button.

Used by:

V1.0 - 10 Sep 2021
V1.1 - 15 Oct 2021, change to TActionType
V1.2 - 23 Nov 2021, enabled clone from file to runFoxClone
V1.3 - 26 Nov 2021, enabled clone drive to drive to runFoxClone
-------------------------------------------------------------------------------}
procedure TClone.NextBtnCK(Sender: TObject);
begin
  case CloneComboBox.ItemIndex of
       0 : if Confirm(acClone,'') then runFoxClone(acClone,''); //no backup filename needed
       1 : if Confirm(acCloneF,RestFname) then runFoxClone(acCloneF,RestFname);
  end;
end; //end procedure TClone.NextBtnCK
{-------------------------------------------------------------------------------
PROCEDURE TCLONE.BOX3CK
Purpose is to check if the selected partition is in the backup set and if not display a
warning message.

Used by:

V1.0 - 28 Sep 2021
-------------------------------------------------------------------------------}
procedure TClone.Box3CK(Sender: TObject);
begin
  if Box3.ItemIndex < 0 then exit;
  DebugInfo(true,'TClone.Box3CK',Box3.Items[Box3.ItemIndex]);
  if not Box3.CheckState[Box3.ItemIndex] then FoxMsg(rCloneMsgTitle,[rCloneMsgStr],mtWarning,[mbOK]);
end; //end TClone.Box3CK

end.

