{ FoxClone
  Copyright (C) 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}
{-------------------------------------------------------------------------------
UNIT - CONFFORM

The objects and methods for the confirm form. To make it easier to rearrange the form dependent
on the function being performed it defines two panels for either a detailed output or a shorter form
output. These are then wrapped up the the TConfForm.

Display may be either Detail > arrow > Info or
                      Info > arrow > Detail

How they are displayed depends on what what has been selected:
        backup - detail panel is displayed first followed by drive panel
        restore - as backup
        clone - drive panel for source, drive panel for destination

Not as pretty as the previous confform in earlier versions of foxclone, but easier to manage - there
is less moving around of the visual objects if switching from a confirm on backup, restore or clone.

Did previously try doing this with TFrames and failed, TPanels used instead.

V1.0 - 09 Dec 2021
-------------------------------------------------------------------------------}
unit confform;
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Dialogs, Graphics, ComCtrls, StdCtrls, ExtCtrls, CheckLst, StrUtils,
  GTKlabel, utilities, il8nstrings, drives, settings;
const
  cWidth = 400;
  cHeight = 300;
  cwBorder = 10;
  chBorder = 10;
  cBtnHeight = 30;
  cDnArrowHeight = 35;
  InfoHeight = 80; //fixed size for the Tpanel, the detail panel is variable for a larger Tchecklistbox
  BtnAdjust = 15; //increase in button height over text height
  ImgWidth = 80; //standard width assumed for all images although drive=80, file=68, partition=64
                 //used for alignment of labels and checklistbox

type
  //TConfirmType = (cBackup, cRestore, cClone, cCloneF)
  //A panel with a title + info = drive (backup and clone) or = filename (restore and clone from file)
  //and checklistbox. The checklistbox is used to display partitions selected for backup, partitions
  //to be restored, content of a backup on clone from file and the contents of a drive on clone
  TDetail = class(TPanel)
    private
      Img : TImage; //the file image icon
      Title : TLabelGTK; //the title, i.e. Drive
      Info : TLabelGTK; //detail on the drive, e.g. xxxx
      ConfParts : TCheckListBox; //list of partitions selected on the drive or backup
      procedure SetScrollBars;
      procedure CheckCK(Sender: TObject);
      procedure CBoxMouseLeave(Sender: TObject);
    public
      constructor Create(TheOwner: TComponent);
  end;
  //A panel with two titles + two info.
  //Backup: Title1 + info = destination drive for backup, Title2 + info = backup filename
  //Restore: Title1 + info = backup filename for a restore, Title2 + info = not used
  //Clone: Title1 = warning message on overwrite, Title2 + info = target drive
  //Clone from file: Title1 = warning message on overwrite, Title2 + info = target drive (same as clone)
  TInfo = class(TPanel) //displays basic info on a drive
    private
      Img : TImage; //the drive image icon
      Title : TLabelGTK; //the title, i.e. Drive
      Info : TLabelGTK; //detail on the drive, e.g. xxxx
      Title2 : TLabelGTK; //the title, i.e. Drive
      Info2 : TLabelGTK; //detail on the drive, e.g. xxxx
    public
      //TFrame does not have OnCreate or OnDestroy events in which private variables can be initialised
      //and released. Override the default constructor and destructor to do this.
      constructor Create(TheOwner: TComponent);
  end;
  TConfForm = class(TForm)
    private
      ConfirmType : TFoxType; //used in resize, set by the confirmXXX procedures
      BtnWidth, BtnHeight : byte; //these were vars in the constructor, moved here for resize
      PathName : string; //used for the backup filename and needed for resize
      Detail : TDetail;
      Drive : TInfo; //need two to cater for drive to drive clone
      Arrow : TImage; //common to all confirm options
      OKBtn,CancelBtn : TButton; //common to all confirm form options
      procedure OKBtnClick(Sender: TObject);
      procedure CancelBtnClick(Sender: TObject);
      procedure ConfirmBackup(FileName : string);
      procedure ConfirmRestore(FileName : string);
      procedure ConfirmClone;
      procedure ConfirmCloneFile(FileName : string);
      procedure ConfReSize(Sender: TObject);
    public
      constructor CreateNew(TheOwner: TComponent);
  end;
//the function called externally to disply the confirm form
function Confirm(Action : TFoxType; FileName : string) : boolean;

implementation

uses
  mainform, //needed for the font
  restore;  //needed to access box3 to determine what partitions have been selected for restore
            //there is now a function that does this
{-------------------------------------------------------------------------------
CONSTRUCTOR TINFO.CREATE
Generates the confirm form

Used by:

V1.0 - 08 Sep 2021
V1.1 - 11 Oct 2021, Font:= MainFM.Font added to pick up changes in font size, wasn't working in
                    TConfForm.
V1.2 - 09 Dec 2021, Problems with visual appearance in the iso, finally fixed by setting the colour
                    of the panel to clForm (after trying many of the other colours). See
                    https://wiki.freepascal.org/Colors#System_colors.
-------------------------------------------------------------------------------}
constructor TInfo.Create(TheOwner: TComponent);
const
  delim = ':';
begin
  inherited Create(TheOwner);
  Width:= cWidth;
  BevelOuter:= bvNone; //turn off borders
  Width:= cWidth;
  Height:= InfoHeight;
  Font:= MainFM.Font; //pick up changes in font size
  Img:= TImage.Create(Self);
  Color:= clForm; //required for visual appearance in the iso
  with Img do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrCenter;
       AnchorSide[akTop].Control:= Self;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Self;
       BorderSpacing.left:= cwBorder;
       AutoSize:= true;
  end;
  Title:= TLabelGTK.Create(Self);
  with Title do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= Self;
       BorderSpacing.top:= chBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Self;
       BorderSpacing.left:= cwBorder + ImgWidth;
       setCaption(rDestinationStr + delim);
  end;
  Info:= TLabelGTK.Create(Self);
  with Info do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= Title;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrRight;
       AnchorSide[akLeft].Control:= Title;
       BorderSpacing.left:= 0;
  end;
  Title2:= TLabelGTK.Create(Self);
  with Title2 do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= Title;
       BorderSpacing.top:= chBorder;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Title;
       BorderSpacing.left:= 0;
       setCaption(rFileStr + delim);
  end;
  Info2:= TLabelGTK.Create(Self);
  with Info2 do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= Title2;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrRight;
       AnchorSide[akLeft].Control:= Title2;
       BorderSpacing.left:= 0;
  end;
end; //end
{-------------------------------------------------------------------------------
CONSTRUCTOR TDETAIL.CREATE
Generates the confirm form

did try a tframe and got a run time error  Resource TDetail not found.


Used by:

V1.0 - 08 Sep 2021
V1.1 - 11 Oct 2021, Font:= MainFM.Font added to pick up changes in font size, wasn't working in
                    TConfForm.
V1.2 - 09 Dec 2021, Problems with visual appearance in the iso, finally fixed by setting the colour
                    of the panel to clForm (after trying many of the other colours). See
                    https://wiki.freepascal.org/Colors#System_colors.
V1.3 - 18 Jan 2022, onmouseleave event added to tchecklistbox
-------------------------------------------------------------------------------}
constructor TDetail.Create(TheOwner: TComponent);
const
  delim = ':';
begin
  inherited Create(TheOwner);
  Width:= cWidth;
  BevelOuter:= bvNone; //turn off borders
  Width:= cWidth;
  Height:= cHeight - InfoHeight - cBtnHeight - 2 * chBorder - cDnArrowHeight;
  Font:= MainFM.Font; //to pick up any changes in font size
  Img:= TImage.Create(Self);
  Color:= clForm; //required for visual appearance in the iso
  with Img do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrCenter;
       AnchorSide[akTop].Control:= Self;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Self;
       BorderSpacing.left:= cwBorder;
       AutoSize:= true;
       Picture.LoadFromFile(PartImgFile);
  end;
  Title:= TLabelGTK.Create(Self);
  with Title do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= Self;
       BorderSpacing.top:= chBorder div 2;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Self;
       BorderSpacing.left:= cwBorder + ImgWidth;
       setCaption(rSourceStr + delim);
  end;
  Info:= TLabelGTK.Create(Self);
  with Info do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= Title;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrRight;
       AnchorSide[akLeft].Control:= Title;
       BorderSpacing.left:= 0;
  end;
  ConfParts:= TCheckListBox.Create(Self);
  with ConfParts do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrBottom;
       AnchorSide[akTop].Control:= Title;
       BorderSpacing.top:= chBorder div 2;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Title;
       BorderSpacing.left:= 0;
       OnClickCheck:= @CheckCK;
       OnMouseLeave:= @CBoxMouseLeave;
       Height:= Self.Height - chBorder - Title.Height;
       Width:= Self.Width - 2 * chBorder - ImgWidth;
  end;
end; //end constructor TDetail.Create
{-------------------------------------------------------------------------------
PROCEDURE TDETAIL.CHECKCK
The event handler for clicking on the checkbox in the detail checklistbox. Cannot disable clicks on
the checkbox element of the box. When the user clicks on the checkbox it changes state, so this simply
sets it back to what it was before as the user should not be able to change the checklistbox in the
confirm form.

Used by:

V1.0 - 07 Sep 2021
-------------------------------------------------------------------------------}
procedure TDetail.CheckCK(Sender: TObject);
begin
  if ConfParts.ItemIndex >= 0 then with ConfParts do Checked[ItemIndex]:= not Checked[ItemIndex];
end; //end procedure TDetail.CheckCK
{-------------------------------------------------------------------------------
PROCEDURE TDETAIL.SETSCROLLBARS
Turns the horizontal scrollbar on/off in the detail tchecklistbox, confparts.

Used by:

V1.0 - 14 Sep 2021
-------------------------------------------------------------------------------}
procedure TDetail.SetScrollBars;
const
  checksize = 40; //width of the checkbox in a TCheckListBox - determined by trial & error
var
  i : byte;
  max : integer = 0;
begin
  for i:= 0 to ConfParts.Count - 1 do
      if GetTextWidth(ConfParts.Items[i],Font) > max then max:= GetTextWidth(ConfParts.Items[i],Font);
  ConfParts.ScrollWidth:= max + checksize;
end; //end procedure TDetail.SetScrollBars
{-------------------------------------------------------------------------------
PROCEDURE TDETAIL.CBOXMOUSELEAVE
This procedure is purely cosmetic - to get a consistent visual appearance across all boxes.

A copy of TBasePanel.BoxMouseLeave

Used by:

V1.0 - 18 Jan 2022
-------------------------------------------------------------------------------}
procedure TDetail.CBoxMouseLeave(Sender: TObject);
var
  i : byte;
begin
  with (Sender as TCheckListBox) do for i:= 0 to Count - 1 do Selected[i]:= false;
end;
{-------------------------------------------------------------------------------
CONSTRUCTOR TCONFFORM.CREATENEW
Generates the confirm form

Used by:

V1.0 - 07 Sep 2021
V1.1 - 14 Sep 2021, BtnWidth, BtnHeight moved into the type definition for access by resize
V1.2 - 11 Oct 2021, was setting Font = MainFM.Font to pick up any changes in font size, was getting
                    alignment problems with a larger font. Moved setting the font to TInfo and TDetail,
                    problem solved.
-------------------------------------------------------------------------------}
constructor TConfForm.CreateNew(TheOwner: TComponent);
begin
  inherited CreateNew(TheOwner);
  Position:= poMainFormCenter; //poScreenCenter;
  Constraints.MinWidth:= cWidth;
  Width:= cWidth;
  Constraints.MinHeight:= cHeight;
  Height:= cHeight;
  OnResize:= @ConfResize;

  Detail:= TDetail.Create(Self);
  with Detail do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrTop;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Self;
       BorderSpacing.left:= 0;
  end;
  Arrow:= TImage.Create(Self);
  with Arrow do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrBottom;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Self;
       BorderSpacing.left:= 3 * cwBorder;
       AutoSize:= true;
       Picture.LoadFromFile(DnArrowImgFile);
  end;
  Drive:= TInfo.Create(Self);
  with Drive do begin
       Parent:= Self;
       AnchorSide[akTop].Side:= asrBottom;
       //AnchorSide[akTop].Control:= Arrow;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Self;
       BorderSpacing.left:= 0;
  end;
  //make the buttons the same width
  if GetTextWidth(rOKstr,Font) >= GetTextWidth(rNoteCancel,Font) then BtnWidth:= GetTextWidth(rOKstr,Font)
  else BtnWidth:= GetTextWidth(rNoteCancel,Font);
  BtnWidth+= cwBorder * 3;
  BtnHeight:= GetTextHeight(rOKstr,Font) + BtnAdjust;
  CancelBtn:= TButton.Create(Self);
  with CancelBtn do begin
       Parent:= Self;
       Width:= BtnWidth;
       Height:= BtnHeight;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= Self;
       BorderSpacing.top:= Self.Height - chBorder - BtnHeight;
       AnchorSide[akLeft].Side:= asrLeft;
       AnchorSide[akLeft].Control:= Self;
       BorderSpacing.left:= Self.Width - 2 * cwBorder - 2 * BtnWidth;
       Caption:= rNoteCancel;
       OnClick:= @CancelBtnClick;
  end;
  OKBtn:= TButton.Create(Self);
  with OKBtn do begin
       Parent:= Self;
       Width:= BtnWidth;
       Height:= BtnHeight;
       AnchorSide[akTop].Side:= asrTop;
       AnchorSide[akTop].Control:= CancelBtn;
       BorderSpacing.top:= 0;
       AnchorSide[akLeft].Side:= asrRight;
       AnchorSide[akLeft].Control:= CancelBtn;
       BorderSpacing.left:= cwBorder;
       Caption:= rOKstr;
       OnClick:= @OKBtnClick;
  end;
end; //end constructor TConfForm.CreateNew
{-------------------------------------------------------------------------------
PROCEDURE TCONFFORM.OKBTNCLICK
Event handler for the OK button, it sets the form modalresult as the modalresult set for the button.

Used by:

V1.0 - 07 Sep 2021
-------------------------------------------------------------------------------}
procedure TConfForm.OKBtnClick(Sender: TObject);
begin
  DebugInfo(true,'TConfForm.OKBtnClick','Okay');
  Self.ModalResult:= mrOK;
end; //end procedure TConfForm.OkBtnClick
{-------------------------------------------------------------------------------
PROCEDURE TCONFFORM.CANCELBTNCLICK
Event handler for the Cancel button, it sets the form modalresult as the modalresult set for the button.

Used by:

V1.0 - 07 Sep 2021
-------------------------------------------------------------------------------}
procedure TConfForm.CancelBtnClick(Sender: TObject);
begin
  DebugInfo(true,'TConfForm.CancelBtnClick','Cancelled');
  Self.ModalResult:= mrCancel;
end; //end procedure TConfForm.CancelBtnClick
{-------------------------------------------------------------------------------
PROCEDURE TCONFFORM.CONFIRMBACKUP
populates the confirm form with the info on the backup

chopstr does not seem to be working with a large font size?

Used by:

V1.0 - 08 Sep 2021
V1.1 - 14 Sep 2021, chopstr added to truncate the backup filename
V1.2 - 13 Oct 2021, const space moved to utilities
V1.3 - 27 Oct 2021, blkadjust removed, drivesize is now in 1k blocks
-------------------------------------------------------------------------------}
procedure TConfForm.ConfirmBackup(FileName : string);
var
  dIndex : shortint;
  i : byte;
  PathWidth : integer;
begin
  Caption:= rConfirmBackupStr;
  PathName:= FileName;
  ConfirmType:= acBackup;
  //load images
  Detail.Img.Picture.LoadFromFile(PartImgFile);
  Drive.Img.Picture.LoadFromFile(FileImgFile);
  //order the panes and arrow
  Detail.AnchorSide[akTop].Control:= Self;
  Arrow.AnchorSide[akTop].Control:= Detail;
  Drive.AnchorSide[akTop].Control:= Arrow;
  //populate the detail panel
  dIndex:= MyDrives.getDrive(acSource); //find the source drive
  if dIndex = -1 then SystemError(1,'TConfForm.ConfirmBackup'); //close foxclone on error
  with MyDrives[dIndex] do
       begin
         //populate the detail panel
         Detail.Info.setCaption(Model + space + SizeStr(DriveSize));
         for i:= 0 to TotalParts - 1 do
             begin
               Detail.ConfParts.Items.Add(Partitions[i].ListStr);
               if Partitions[i].selected then Detail.ConfParts.Checked[i]:= true;
             end;
       end;
  Detail.SetScrollBars;
  //populate the info panel
  dIndex:= MyDrives.getDrive(acTarget); //find the source drive
  if dIndex = -1 then SystemError(6,'TConfForm.ConfirmBackup'); //close foxclone on error
  with MyDrives[dIndex] do Drive.Info.setCaption(Model + space + SizeStr(DriveSize));
  PathWidth:= Self.Width - cwBorder - ImgWidth - Drive.Title2.Width;
  Drive.Info2.setCaption(ChopStr(PathName,Font,PathWidth));
end; //end procedure TConfForm.ConfirmBackup
{-------------------------------------------------------------------------------
PROCEDURE TCONFFORM.CONFIRMRESTORE
Populates the confirm form with the info on restore. The order of the form is:

    info panel with file img and showing the backup filename.
    downarrow.
    detail panel with the partition image and a checklistbox of the partitions to be restored.

The procedure has to load the appropriate image files and re-order the panels. Then populate the
checklistbox from RDrive. Status of partitions to be restored is determined by the check state of the
items in Box3 of the restore tab.

Filename = the name of the backup filename.

Used by:

V1.0 - 09 Sep 2021
V1.1 - 14 Sep 2021, chopstr added to truncate the restore filename
V1.2 - 13 Nov 2021, missing the partition table on the confform checklistbox, added
-------------------------------------------------------------------------------}
procedure TConfForm.ConfirmRestore(FileName : string);
const
  delim = ':';
var
  i : byte;
  dIndex : shortint;
  PathWidth : integer;
begin
  Caption:= rConfirmRestoreStr;
  PathName:= FileName;
  ConfirmType:= acRestore;
  //load images
  Detail.Img.Picture.LoadFromFile(PartImgFile);
  Drive.Img.Picture.LoadFromFile(FileImgFile);
  //order the panes and arrow
  Drive.AnchorSide[akTop].Control:= Self;
  Drive.AnchorSide[akTop].Side:= asrTop;
  Arrow.AnchorSide[akTop].Control:= Drive;
  Detail.AnchorSide[akTop].Side:= asrBottom;
  Detail.AnchorSide[akTop].Control:= Arrow;
  //hide stuff not needed - hide the file label and reuse title label for file info
  Drive.Title.setCaption(rFromFileStr + delim);
  Drive.Title2.Visible:= false;
  Drive.Title.BorderSpacing.top:= 3 * chBorder;
  //set the restore filename
  PathWidth:= Self.Width - cwBorder - ImgWidth - Drive.Title.Width;
  Drive.Info.setCaption(ChopStr(PathName,Font,PathWidth));
  //change the title on the detail panel
  Detail.Title.setCaption(rToDriveStr + delim);
  //get the drive model name
  Detail.Info.setCaption(RDrive.Model);
  //populate the checklistbox with partitions
  //first item is the partition table
  Detail.ConfParts.Items.Add(rPartTableStr);
  for i:= 0 to RDrive.TotalParts - 1 do with RDrive do Detail.ConfParts.Items.Add(Partitions[i].ListStr);
  //overwrite the drive, e.g. sda, just in case it has changed - at this point the user has already
  //agreed if the mountpoint for the drive to restore to has changed, say, from sda to sdd.
  dIndex:= MyDrives.getDrive(acTarget); //find the source drive  NEED TO CHECK THAT ACTARGET HAS BEEN SET CORRECTLY
  if dIndex = -1 then SystemError(12,'TConfForm.ConfirmRestore'); //close foxclone on error
  for i:= 1 to Detail.ConfParts.Count - 1 do //Note 1st item is the part table so start the loop at 1
      with Detail.ConfParts do Items[i]:= replacestr(Items[i],RDrive.MountPt,MyDrives[dIndex].MountPt);
  //selecting or deselecting a partition for restore does not change the state of the partition
  //in RDrive - the status of selected in RDrive indicates that a partition does or does not
  //have a backup. To determine the status of the partitions to be restored, the check status of the
  //contents of box3 on the restore tab must be read.
  for i:= 0 to RestoreTab.PartBox.Items.Count - 1 do Detail.ConfParts.Checked[i]:= RestoreTab.RestoreSelect(i);
  Detail.SetScrollBars;
end; //procedure TConfForm.ConfirmRestore
{-------------------------------------------------------------------------------
PROCEDURE TCONFFORM.CONFIRMCLONE
Populates the confirm form with the info on clone drive to drive. The order of the panels on the form
is unchanged from a backup, but the image files are different

Used by:

V1.0 - 10 Sep 2021
V1.1 - 13 Oct 2021, const space moved to utilities
V1.2 - 27 Oct 2021, blkadjust removed, drivesize is now in 1k blocks
V1.3 - 27 Nov 2021, part table added to the confirm checklistbox for consistency with restore, etc.
-------------------------------------------------------------------------------}
procedure TConfForm.ConfirmClone;
const
  delim = ':';
var
  i : byte;
  dIndex : shortint;
begin
  Caption:= rConfirmCloneStr;
  ConfirmType:= acClone;
  //load images
  Detail.Img.Picture.LoadFromFile(DriveImgFile);
  Drive.Img.Picture.LoadFromFile(DriveImgFile);
  //order the panes and arrow
  Detail.AnchorSide[akTop].Control:= Self;
  Arrow.AnchorSide[akTop].Control:= Detail;
  Arrow.BorderSpacing.left:= 4 * cwBorder; //looks better
  Drive.AnchorSide[akTop].Control:= Arrow;
  //populate the detail panel
  Detail.Title.setCaption(rSourceStr + delim);
  dIndex:= MyDrives.getDrive(acSource); //find the source drive
  if dIndex = -1 then SystemError(19,'TConfForm.ConfirmClone'); //close foxclone on error
  with MyDrives[dIndex] do Detail.Info.setCaption(Model + space + SizeStr(DriveSize));
  //populate the checklistbox
  //for consistency add the partition table as first item
  Detail.ConfParts.Items.Add(rPartTableStr);
  Detail.ConfParts.Checked[0]:= true;
  for i:= 0 to MyDrives[dIndex].TotalParts - 1 do with Detail.ConfParts do //populate the checklistbox
      begin
        Items.Add(MyDrives[dIndex].Partitions[i].ListStr);
        Checked[i+1]:= true; //because part table is item 0
      end;
  //set scrollbars
  Detail.SetScrollBars;
  //populate the info panel
  Drive.Title.setCaption(rCloneWarnStr);
  Drive.Title.Font.Color:= ClRed;
  Drive.Info.Visible:= false;
  Drive.Title2.setCaption(rTargetStr + delim);
  dIndex:= MyDrives.getDrive(acTarget); //find the source drive
  if dIndex = -1 then SystemError(24,'TConfForm.ConfirmClone'); //close foxclone on error
  with MyDrives[dIndex] do Drive.Info2.setCaption(Model + space + SizeStr(DriveSize));
end; //procedure TConfForm.ConfirmClone
{-------------------------------------------------------------------------------
PROCEDURE TCONFFORM.CONFIRMCLONEFILE
Populates the confirm form with the info on clone from backup file to drive.

Nearly identical to ConfirmClone, changes are filename instead of source drive and the file image instead
of drive image

Used by:

V1.0 - 10 Sep 2021
V1.1 - 14 Sep 2021, chopstr added to truncate the restore filename
V1.2 - 13 Oct 2021, const space moved to utilities
V1.3 - 27 Oct 2021, blkadjust deleted, drivesize is now 1k blocks
V1.4 - 24 Nov 2021, partition table added to the checklistbox for consistency with restore and the
                    listbox showing on the progress form.
-------------------------------------------------------------------------------}
procedure TConfForm.ConfirmCloneFile(FileName : string);
const
  delim = ':';
var
  i : byte;
  dIndex : shortint;
  PathWidth : integer;
begin
  Caption:= rConfirmCloneFileStr;
  PathName:= FileName;
  ConfirmType:= acCloneF;
  //load images
  Detail.Img.Picture.LoadFromFile(FileImgFile);
  Drive.Img.Picture.LoadFromFile(DriveImgFile);
  //order the panes and arrow
  Detail.AnchorSide[akTop].Control:= Self;
  Arrow.AnchorSide[akTop].Control:= Detail;
  Arrow.BorderSpacing.left:= 4 * cwBorder; //looks better
  Drive.AnchorSide[akTop].Control:= Arrow;
  Detail.Title.setCaption(rFileStr + delim);
  PathWidth:= Self.Width - cwBorder - ImgWidth - Detail.Title.Width;
  Detail.Info.setCaption(ChopStr(PathName,Font,PathWidth));
  dIndex:= MyDrives.getDrive(acSource); //find the source drive
  if dIndex = -1 then SystemError(19,'TConfForm.ConfirmCloneFile'); //close foxclone on error
  //populate the checklistbox
  Detail.ConfParts.Items.Add(rPartTableStr);
  Detail.ConfParts.Checked[0]:= true;
  for i:= 0 to RDrive.TotalParts - 1 do with Detail.ConfParts do //populate the checklistbox
      begin
        Items.Add(RDrive.Partitions[i].ListStr);
        Checked[i+1]:= true; //i+1 because partition table is item 0
      end;
  //set scrollbars
  Detail.SetScrollBars;
  //populate the info panel
  Drive.Title.setCaption(rCloneWarnStr);
  Drive.Title.Font.Color:= ClRed;
  Drive.Info.Visible:= false;
  Drive.Title2.setCaption(rTargetStr + delim);
  dIndex:= MyDrives.getDrive(acTarget); //find the source drive
  if dIndex = -1 then SystemError(24,'TConfForm.ConfirmCloneFile'); //close foxclone on error
  with MyDrives[dIndex] do Drive.Info2.setCaption(Model + space + SizeStr(DriveSize));
end; //procedure TConfForm.ConfirmCloneFile
{-------------------------------------------------------------------------------
PROCEDURE TCONFFORM.RESIZE
Resizes the confirm form and uses chopstr to adjust the length of the backup filename shown on the form.

Used by:

V1.0 - 14 Sep 2021
V1.1 - 15 Oct 2021, multiple if statements changed to single case statement
-------------------------------------------------------------------------------}
procedure TConfForm.ConfReSize(Sender: TObject);
var
  PathWidth : integer;
begin
  //set height/width of the detail panel
  Detail.Height:= Height - InfoHeight - cBtnHeight - 2 * chBorder - cDnArrowHeight;
  Detail.Width:= Width;
  //set height/width of the confparts tchecklistbox
  Detail.ConfParts.Height:= Detail.Height - chBorder - Detail.Title.Height;
  Detail.ConfParts.Width:= Detail.Width - 2 * chBorder - ImgWidth;
  //set width of the drive panel, height is fixed
  Drive.Width:= Width;
  //set scroll bars
  Detail.SetScrollBars;
  //reset the filenames to show more/less - backup
  case ConfirmType of
       acBackup :  begin
                     PathWidth:= Width - cwBorder - ImgWidth - Drive.Title2.Width;
                     Drive.Info2.setCaption(ChopStr(PathName,Font,PathWidth));
                   end;
       acRestore : begin
                     PathWidth:= Width - cwBorder - ImgWidth - Drive.Title.Width;
                     Drive.Info.setCaption(ChopStr(PathName,Font,PathWidth));
                   end;
       acCloneF :  begin
                     PathWidth:= Width - cwBorder - ImgWidth - Detail.Title.Width;
                     Detail.Info.setCaption(ChopStr(PathName,Font,PathWidth));
                   end;
  end;
  CancelBtn.BorderSpacing.top:= Height - chBorder - BtnHeight;
  CancelBtn.BorderSpacing.left:= Width - 2 * cwBorder - 2 * BtnWidth;
end; //end procedure TConfForm.ConfReSize
{-------------------------------------------------------------------------------
FUNCTION CONFIRM
Provides access to the confirm form from elsewhere. Generates the confirm form.

Parameters:
     cAction - used to set up the form and sort out what to display by calling the relevant procedure.
     FileName - full path name of the file to save backup or restore from etc. This is assigned to
     PathName, defined in the type defn for TConfForm. This is so that it is available to the resize
     procedure.

The function returns true if the user clicks the Okay button false otherwise.

Used by:

V1.0 - 07 Sep 2021
V1.1 - 05 Oct 2021, modded to run partclone on backup ok
V1.2 - 15 Oct 2021, passed param change to Action : TFoxType
-------------------------------------------------------------------------------}
function Confirm(Action : TFoxType; FileName : string) : boolean;
var
  ConfForm : TConfForm;
  Outcome : shortint;
begin
  ConfForm:= TConfForm.CreateNew(MainFm); //make the owner mainfm so it will freed up on closure
  case Action of
       acBackup  : ConfForm.ConfirmBackup(FileName);
       acRestore : ConfForm.ConfirmRestore(FileName);
       acClone   : ConfForm.ConfirmClone;
       acCloneF  : ConfForm.ConfirmCloneFile(FileName);
  end;
  Outcome:= ConfForm.showmodal;
  ConfForm.free;
  if Outcome = mrOk then Confirm:= true else Confirm:= false;
end;

end.

