{ FoxClone
  Copyright (C) 2021 Andy Hardwick
  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 51 Franklin Street - Fifth Floor,
  Boston, MA 02110-1335, USA.
}
{-------------------------------------------------------------------------------
UNIT - DRIVES

The data structures used in foxclone to describe a system are:

    TSystem - a TObjectList of all the drives in a PC.

    TDrive - one for each drive in the system, defines the data held on a drive, e.g. name, size,
    partition table type and the methods to access/change those data.

    TPartList - one of the data attributes of a TDrive is a TPartList, a list of all the partitions
    on a drive.

    TPart - one for each partition on a drive, defines the data held on a partition, e.g. size,
    file system, label, used space and the methods to access/change those data.

This unit defines the data structures and methods for a TSystem and TDrive.

V1.0 - 24 Feb 2021
-------------------------------------------------------------------------------}
unit drives;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Contnrs, StrUtils, Dialogs, Controls, process, partitions, utilities, il8nstrings,
  fdialog, Info;

type
  //acSource = drive selected as source drive, acTarget = drive selected as target drive
  //for backup acSource is the drive to be backed up and acTarget is where the backup will be stored
  //for restore acSource is the drive containing the backup and acTarget is the destination.
  TActionType = (acNone, acSource, acTarget);

  TDrive = class(TObject)
    private
      Device     : string;  //e.g. sda
      Selected   : TActionType; //set by checklistboxes
      dBoot       : boolean; //when true is the boot drive = cannot be backed up or restored
      dModel      : string;  // from parted e.g. ATA WDC WDS240G2G0A
      dLogical : integer; // from parted drive logical sector size in bytes, duplicates sfdisk
      dFlags : string; // from parted disk flags, yet see it populated, not implemented
      pTableType : string;  // from parted msdos or gpt
      dSerial     : string;  //the serial number of the drive
      PartPrefix : string;  // empty for sata, for nvme or SD card = p
      dFirstPart  : integer; //start of the 1st partition in sectors, needed for grub
      Size       : int64;   //size of drive in blocks
      DriveUUID    : string;  //from sfdisk
      sfFirstLBA : shortint; //from sfdisk on a gpt drive would be 34
      sfLastLBA : int64; //from sfdisk
      PartCount : integer; //populated when TSystem.Initialise is run
      Parts : TPartList;
    public
      property MountPt : string read Device write Device;
      property isBoot : boolean read dboot; //returns true if the boot drive
      property Prefix : string read PartPrefix write PartPrefix;
      property TotalParts : integer read PartCount write PartCount;
      property pTable : string read pTableType write pTableType;
      property Partitions : TPartList read Parts write Parts;
      property setAction : TActionType read selected write selected;
      property Model : string read dModel write dModel;
      property Serial : string read dSerial write dSerial;
      property FirstLBA : shortint read sfFirstLBA write sfFirstLBA;
      property LastLBA : int64 read sfLastLBA write sfLastLBA;
      property FirstPart : integer read dFirstPart write dFirstPart;
      property DriveSize : int64 read Size;
      property DriveSector : integer read dLogical write dLogical;
      constructor create;
      procedure getDriveInfo;
      function ListStr : string; //generates a string for display in checklist boxes
      function HintStr : string; //generates a string for hints on a checklist box showing drive info
      procedure Print(var OutF : Textfile);
      function isLoop : boolean; //true if the drive is a loop device
      function isBlank : boolean; //true is the drive is blank or has an unknown partition table
      procedure ReadsfDisk(sfFileName : string);
      function CheckPartTable(Fname : string) : boolean;
      function GrubBackup(GrubFile : string = '') : boolean;
      function GrubRestore(GrubFile : string = '') : boolean;
      function GrubClone : boolean;
      function sfdisk(Action : TFoxType; sfFile : string = '') : boolean;
      function PartProbe : boolean;
  end;

  TSystem = class(TObjectList)
    private
      function GetItems(aIndex: Integer): TDrive;
      procedure SetItems(aIndex: Integer; aDrive: TDrive);
    public
      property Items[Index: Integer]: TDrive read GetItems write SetItems; default;
      function OSprobe : byte; //find operating systems
      procedure FindBoot; //returns index of boot drive
      function FindName(DriveStr: String): shortint; //returns index of drive based on device = sda, etc.
      function FindModel(ModelName : string; SerialNo : string) : shortint; //find based on model name
      function notHintStr : string; //returns a string of drives excluded for backup
      function getDrive(ActionType : TActionType) : shortint; //returns index of drive
      procedure unSetAll(ActionType : TActionType); //sets all drives to acNone & parts selected to false
      function CheckSpace : boolean; //checks space on drive acTarget against used on drive acSource
      function isOS(PartStr : string) : boolean; //returns true if a partition has an OS on it
      procedure SetFat32Split(OnOff : boolean); //checks and sets split for backup to fat32 partition

  end;

var
  MyDrives : TSystem;
  Drive: TDrive; //TDrives created and pointed to in MyDrives
  RDrive : TDrive; //a TDrive populated from a backup file on a restore

implementation

uses
  settings;
{-------------------------------------------------------------------------------
FUNCTION TSYSTEM.GETITEMS
A wrapper so that instead of accessing an element in a TDrive with:
  TDrive(MyDrives[i]).mountpt)
It can be accessed with:
  MyDrives[i].mountpt

It replaces the default TObjectList.Items[Index: Integer] property of a TObjectList

Used by:

V1.0 - 24 Feb 2021
-------------------------------------------------------------------------------}
function TSystem.GetItems(aIndex: Integer): TDrive;
begin
  result:= TDrive(get(aIndex));
end; //end function TSystem.GetItems
{-------------------------------------------------------------------------------
PROCEDURE TSYSTEM.SETITEMS
A wrapper so that instead of accessing an element in a TDrive with:
  TDrive(MyDrives[i]).mountpt)
It can be accessed with:
  MyDrives[i].mountpt

It replaces the default TObjectList.Items[Index: Integer] property of a TObjectList

  Used by:

V1.0 - 24 Feb 2021
-------------------------------------------------------------------------------}
procedure TSystem.SetItems(aIndex: Integer; aDrive: TDrive);
begin
  put(aIndex,aDrive);
end; //end procedure TSystem.SetItems
{-------------------------------------------------------------------------------
FUNCTION TSYSTEM.FINDNAME
Seaches the system to find the matching drive.

Parameters:
    DriveStr : string of the form 'sda1 and other text'

The function searches for a match = device in DriveStr. It returns the index of the drive or if not
found returns -1. For a match DriveStr MUST start with the drive reference = device in TDrive.

Only a small number of drives, so searching from start to finish not a big overhead.

Used by:

V1.0 - 27 Feb 2021
-------------------------------------------------------------------------------}
function TSystem.FindName(DriveStr: String): shortint;
var
  i : shortint;
  found : boolean = false;
begin
  for i:= 0 to self.Count - 1 do
      if pos(self[i].device,DriveStr) = 1 then
         begin
           found:= true;
           FindName:= i;
           break;
         end;
  if not found then FindName:= -1;
end; //end function TSystem.FindbyName
{-------------------------------------------------------------------------------
FUNCTION TSYSTEM.FINDMODEL
Seaches the system to find the matching drive.

Parameters:
    ModelName : string containing the model name of the drive to be found
    SerialNo : string containing the serial number

The function searches for a match on ModelName and SerialNo. It returns the index of the drive or if not
found returns -1. Older versions of foxclone did not save the serial number on a backup so if the
serial number is an empty string, it searches on ModelName only.

Only a small number of drives, so searching from start to finish not a big overhead.

If the drive is not found an error message is displayed.

Used by:

V1.0 - 26 Apr 2021
V1.1 - 11 May 2021, FoxDialog replaced by FoxMsg
V1.2 - 13 Oct 2021, const space moved to utilities
V1.3 - 19 Feb 2022, debug info added
-------------------------------------------------------------------------------}
function TSystem.FindModel(ModelName : string; SerialNo : string) : shortint;
var
  i : shortint;
begin
  DebugInfo(false,'TSystem.FindModel',ModelName,SerialNo);
  FindModel:= -1; //default not found
  if SerialNo = '' then for i:= 0 to Count - 1 do
     begin //need to nest begin-end as without it else clause belongs to last if statement
       if Self[i].Model = ModelName then
          begin
            FindModel:= i;
            break;
          end
     end
  else for i:= 0 to Count - 1 do
     if (Self[i].Model = ModelName) and (Self[i].Serial = SerialNo) then
        begin
          FindModel:= i;
          break;
        end;
  if FindModel = -1 then
     if SerialNo = '' then FoxMsg(rDriveNotFoundStr,[rDriveFailStr,ModelName],mtError,[mbOk])
     else FoxMsg(rDriveNotFoundStr,[rDriveFailStr,ModelName + LF + rSerialStr + space + SerialNo],mtError,[mbOk]);
end; //end function TSystem.FindModel
{-------------------------------------------------------------------------------
FUNCTION TSYSTEM.OSPROBE
Searches for operating systems using os-prober.

Must be called AFTER all other information on the system has been collected using
TDrive.getInfo and TPart.getInfo. So that the data structures are in place to store the results.

Populates MyDrives[x].partitions[y].OpSys with operating systems found.

Returns the number of operating systems found.

Used by:

V1.0 - 27 Feb 2021
V1.1 - 09 Dec 2021, running in a VM was getting a list index out of bounds, if statement added to the
                    for loop.
V1.2 - 28 Dec 2021, was referring to mydrives, replaced with self
-------------------------------------------------------------------------------}
function TSystem.OSprobe : byte;
var
  OSlist : TStringList;
  i : byte;
  dindex,pindex : integer;
begin
  OSlist:= TStringList.create;
  getText('pkexec os-prober',OSList);
  if OSList.Count > 0 then for i:= 0 to OSList.Count - 1 do if OSList[i] <> '' then
      begin
        OSlist[i]:= replacestr(OSlist[i],'/dev/','');
        dindex:= Self.FindName(OSlist[i]);
        if dindex >= 0 then
            begin
              pindex:= Self[dindex].parts.FindName(OSlist[i]);
              if pindex >= 0 then
                  begin
                    OSlist[i]:= copy(OSlist[i],pos(':',OSlist[i])+1,99);
                    OSlist[i]:= copy(OSlist[i],1,pos(':',OSlist[i])-1);
                    Self[dindex].parts[pindex].OpSys:= OSlist[i];
                  end;
              //else writeln('Internal error: Partition not found for OS-prober : ',OSlist[i]);
            end;
        //else writeln('Internal error: Drive not found for OS-prober : ',OSlist[i]);
      end;
  OSprobe:= OSList.Count;
  OSlist.free;
end;
{-------------------------------------------------------------------------------
PROCEDURE TSYSTEM.FINDBOOT
Searches for the boot drive.

First seaches all partitions for one mounting as /. If that fails then runs df where output would
typically be:

Filesystem     1K-blocks   Used Available Use% Mounted on
udev             3876000      0   3876000   0% /dev
tmpfs             782704   1172    781532   1% /run
/dev/sdd          619520 619520         0 100% /cdrom
/dev/loop0        532608 532608         0 100% /rofs
/cow             3913504  28644   3884860   1% /
tmpfs            3913504      0   3913504   0% /dev/shm
tmpfs               5120      0      5120   0% /run/lock
tmpfs            3913504      0   3913504   0% /sys/fs/cgroup
tmpfs            3913504      4   3913500   1% /tmp
tmpfs             782700      4    782696   1% /run/user/1000

When booting from a usb stick it gets mounted as /cdrom. This is grepped and parsed to find the drive.

Sets MyDrives[x].dboot = true

Need to check what it shows when booting from ventoy.

When using blkid to get the label, big problems. Run iso in VB, okay, run iso from ventoy not okay.
Run blkid from terminal in ventoy, no output, run with pkexec in terminal get output. The run blkid
on its own and get output. Put pkexec in cmd string below redo the iso and boot, no work???

Change terminal command to lsblk -iPo name,label

Booting from ventoy - /dev/mapper/ventoy gets mounted at /cdrom so a check for /cdrom has to
differentiate between a standard usb stick and ventoy.

To find find the drive with ventoy, use the ventoydump utility, see procedure FindVentoy.

Used by:

V1.0 - 27 Feb 2021
V1.1 - 10 Jan 2022, modified to search for booting off a ventoy stick. This has two parts on it, exfat
                    containing the isos and a fat16 efi part. Ventoy uses a virtual device with
                    /dev/mapper which uses the exfat part and does not show / mounted on any physical
                    partition. The only clue is the exfat part labelled Ventoy.
V1.2 - 24 Jan 2022, code added to check for ventoy.
V1.3 - 02 Feb 2022, modified to vtoydump to find the ventoy boot drive if booting from ventoy
-------------------------------------------------------------------------------}
procedure TSystem.FindBoot;
const
  VentoyMountStr = '/dev/mapper/ventoy';
var
  i, j, dindex : shortint;
  Found : boolean = false;
  BootStr : string;
begin
  //search for a partition mounted as root
  for i:= 0 to Self.Count -1 do
      for j:= 0 to Self[i].partitions.Count - 1 do
          if Self[i].partitions[j].boot then
              begin
                dindex:= i;
                Found:= true;
                break;
              end;
  if Found then
     begin
       Self[dindex].dBoot:= true;
       exit;
     end;
  //not booting from a normal drive, look for usb boot
  BootStr:= getStr('pkexec df | grep /cdrom$'); // $ searches at the end of the line
  //need to check if booting ventoy, it mounts the iso, /cdrom at /dev/mapper/ventoy
  if pos('/dev/',BootStr) = 1 then
     begin
       if pos(VentoyMountStr,BootStr) = 0 then //not booting from ventoy
          begin
            BootStr:= replacestr(BootStr,'/dev/','');
            dindex:= Self.FindName(BootStr);
            if dindex >= 0 then Self[dindex].dBoot:= true;
            exit;
          end;
       BootStr:= FindVentoy; //returns drive if found, e.g. sdc
       if BootStr = '' then exit; //not found
       dindex:= Self.FindName(BootStr);
       if dindex >= 0 then Self[dindex].dBoot:= true;
     end;
end; //end procedure TSystem.FindBoot;
{-------------------------------------------------------------------------------
FUNCTION TSYSTEM.ISOS
Returns true if a partition has an operating system on it (as found by osprober). It searches through
all the partitions in the system looking for a partition where OS <> ''.

Parameters:
    PartStr - a string taken from a CheckListBox containing, e.g. sda1. the findname methos of TDrive
              TPartition are used to find the partition and then check if an OS is present.

Used by:

V1.0 - 10 Mar 2021
-------------------------------------------------------------------------------}
function TSystem.isOS(PartStr : string) : boolean;
var
  dIndex : shortint = -1;
  pIndex : shortint = -1;
begin
  isOS:= false;
  //find the drive
  dIndex:= Self.FindName(PartStr);
  if dIndex > -1 then //find the partition
     begin
       pIndex:= Self[dIndex].parts.FindName(PartStr);
       if pIndex > -1 then if Self[dIndex].parts[pIndex].OpSys <> '' then isOS:= true;
     end;
end;
{-------------------------------------------------------------------------------
FUNCTION TSYSTEM.NOTHINTSTR
Returns a string containg details on the drives that cannot be backed up. These are:

        the boot drive
        a drive with no partition table (a loop device), but a valid filesystem
        a blank drive (or a drive with an unknown partition table type)

The string is displayed as a hint when the user clicks in a blank area of box1 on backup.

Note - will not include boot drives when booting from ventoy or other multi-boot usb stick solutions.
there is no easy way that is not specific to each application to determine the boot drive. Policy
decision not to support this in foxclone - where does it stop?

Used by:

V1.0 - 10 Mar 2021
-------------------------------------------------------------------------------}
function TSystem.notHintStr : string; //returns a string of drives excluded for backup
const
  delim = ' : ';
  LF2 = LF + LF;
var
  i : byte;
begin
  notHintStr:= rnotHintStr + delim;
  for i:= 0 to self.count - 1 do with self[i] do
      begin
        if isBoot then notHintStr+= LF2 + Device + delim + rnotHintBootStr;
        if isLoop then notHintStr+= LF2 + Device + delim + rnotHintLoopStr;
        if isBlank then notHintStr+= LF2 + Device + delim + rnotHintBlankStr;
      end;
  notHintStr+= LF + ' ';
end; //end function TSystem.notHintStr
{-------------------------------------------------------------------------------
FUNCTION TSYSTEM.GETDRIVE
Searches drives in the system for the drive with selected = ActionType and returns the index of
the drive in the list of drives.

Parameters:
    ActionType - the action to search on, typically acSource or acTarget

Returns -1 if the drive is not found.

Used by:

V1.0 - 10 Mar 2021
-------------------------------------------------------------------------------}
function TSystem.getDrive(ActionType : TActionType) : shortint;
var
  i : shortint;
begin
  getDrive:= -1;
  for i:= 0 to self.Count - 1 do if self[i].Selected = ActionType then getDrive:= i;
end; //end TSystem.getDrive
{-------------------------------------------------------------------------------
PROCEDURE TSYSTEM.UNSETALL
Resets all drives so that selected = acNone and for each partition sets selected = false

Parameters:
    ActionType - when this is acSource or acTarget sets the drive to acNone and the partitions on the
    drive to selected = false. When this is acNone resets everything, i.e. all drives are set acNone and
    all partitions are set selected = false.

Used by:

V1.0 - 10 Mar 2021
V1.1 - 29 Mar 2021 modified so that it unsets everything if ActionType is acNone.
-------------------------------------------------------------------------------}
procedure TSystem.unSetAll(ActionType : TActionType);
var
  i,j : shortint;
begin
  for i:= 0 to Self.Count - 1 do
      if ActionType = acNone then
         begin
           for j:= 0 to self[i].PartCount - 1 do Self[i].Partitions[j].Selected:= false;
           Self[i].Selected:= acNone;
         end
      else if Self[i].Selected = ActionType then
           begin
             for j:= 0 to self[i].PartCount - 1 do Self[i].Partitions[j].Selected:= false;
             Self[i].Selected:= acNone;
           end;
end; //end procedure TSystem.unSetAll
{-------------------------------------------------------------------------------
FUNCTION TSYSTEM.CHECKSPACE
Searches the drives for the drive set acSource and then adds up the total sectors on the partitions
where selected = true.

Seaches the drives to find the drive set acTarget and then for the partition where selected = true, gets
the free space.

Compares the space required against the space available and returns true if there is enough space. If
not enough space, checks to see if compression is set and does it make a difference.

Most of the code is associated with putting together the message strings for the message dialogs.

Used by:

V1.0 - 24 Mar 2021
V1.1 - 11 May 2021, FoxDialog replaced by FoxMsg
V1.2 - 28 Oct 2021, changes as a result of removing blkadjust, psize, pused and drive size are in
                    blocks
V1.3 - 10 Dec 2021, checks on totalparts > 0 added
V1.4 - 05 Jan 2022, error messages now display in MB not blocks
-------------------------------------------------------------------------------}
function TSystem.CheckSpace : boolean;
const
  aDeststr = '[destination]';
  CompFactor = 0.7; //reduction in space required due to compression - a judgement
  bullet = '•   ';  //used in the message dialogs
var
  i : shortint;
  sdIndex : shortint = -1; //index for source drive
  tdIndex : shortint = -1; //index for the target drive
  tpIndex : shortint = -1; //index for target partition
  SourceBlocks : int64; //number of blocks to backup
  CompBlocks : int64; //number of sectors to backup with compression
  FreeBlocks : int64;   //free space on the target partition
  Msg1, Msg2, Msg3 : string;
begin
  CheckSpace:= false;
  //find the source drive (selected=acSource) and then get space used.
  for i:= 0 to self.Count - 1 do if self[i].Selected = acSource then sdindex:= i;
  SourceBlocks:= 0;
  if sdIndex = -1 then SystemError(8,'TSystem.CheckSpace');
  //check on total parts added
  with self[sdIndex] do if TotalParts > 0 then for i:= 0 to TotalParts - 1 do
      if Parts[i].selected then SourceBlocks+= Parts[i].Used;
  //Is compression switched on in settings? Work out sectors required with compression.
  if (Defaults.bSettings = seComp) or (Defaults.bSettings = seBoth) then
     CompBlocks:= round(CompFactor * SourceBlocks) else CompBlocks:= SourceBlocks;
  //find the target drive (selected=acTarget) and target partition, then get free space
  if self.Count > 0 then for i:= 0 to self.Count - 1 do if self[i].Selected = acTarget then tdIndex:= i;
  if tdIndex = -1 then SystemError(9,'TSystem.CheckSpace');
  //check on totalparts added
  with self[tdIndex] do if TotalParts > 0 then for i:= 0 to TotalParts - 1 do if Partitions[i].selected then tpIndex:= i;
  if tpIndex = -1 then SystemError(10,'TSystem.CheckSpace');
  with self[tdIndex].Partitions[tpIndex] do FreeBlocks:= Size - Used;
  //Now find out if the backup will fit on the destination with/without compression
  if (FreeBlocks - SourceBlocks) > 0 then CheckSpace:= true //backup will fit
  else
    if (FreeBlocks - CompBlocks) > 0 then //compressed backup will fit
      begin
        Msg1:= bullet + rSpaceReqdStr + FormatMB(SourceBlocks);
        Msg2:= bullet + rCompReqdStr + FormatMB(CompBlocks);
        Msg3:= bullet + rSpaceAvailStr + FormatMB(FreeBlocks);
        with self[tdIndex].Partitions[tpIndex] do Msg3:= ReplaceStr(Msg3,aDeststr,Drive + Number);
        if FoxMsg(rBackTabStr,[rCompFitStr,Msg1,Msg2,Msg3,rProceedStr],mtWarning,[mbOK,mbCancel]) = mrOK
        then CheckSpace:= true;
      end
    else //wont fit
       begin
         Msg1:= rNoSpaceStr;
         with self[tdIndex].Partitions[tpIndex] do Msg1:= ReplaceStr(Msg1,aDeststr,Drive + Number);
         Msg2:= bullet + rSpaceReqdStr + FormatMB(SourceBlocks);
         Msg3:= bullet + rSpaceAvailStr + FormatMB(FreeBlocks);
         with self[tdIndex].Partitions[tpIndex] do Msg3:= ReplaceStr(Msg3,aDeststr,Drive + Number);
         FoxMsg(rBackTabStr,[Msg1,Msg2,Msg3],mtError,[mbOK]);
       end;
end; //end function TSystem.CheckSpace
{-------------------------------------------------------------------------------
CONSTRUCTOR TDRIVE.CREATE
Simply initialises all the variables in TDrive and creates a TPartList

Used by:

V1.0 - 24 Feb 2021
V1.1 - 06 Mar 2021 - decided to record all info from sfdisk and convert most internal parameters
                     to sectors not blocks.
V1.2 - 27 Oct 2021 - blkadjust removed, default unit for all sizes now 1k blocks
-------------------------------------------------------------------------------}
constructor TDrive.create;
begin
  inherited create;
  selected:= acNone;
  dBoot:= false;
  dModel:= '';
  dlogical:= 512; //default, is overwritten
  dflags:= '';
  dSerial:= '';
  device:= '';
  partprefix:= '';
  dFirstPart:= 2048; //what it normally is - in sectors
  size:= 0;
  ptabletype:= '';
  DriveUUID:= '';
  sfFirstLBA:= 0;
  sfLastLBA:= 0;
end; //end constructor TDrive.create
{-------------------------------------------------------------------------------
PROCEDURE TDRIVE.GETDRIVEINFO
Populates a Tdrive with drive information.

Rewritten to use the info unit and the SysInfo object. GetDriveInfo populates a TDrive by calling the
methods in SysInfo. For information that is parsed from the output of parted or sfdisk, this
information is already in memory as TStringLists, so a lot faster.

Used by:

V1.0 - 24 Feb 2021
V1.1 - 04 Mar 2021 modified to deal with loop devices or blank drives
V1.2 - 06 Mar 2021 modified to pull all information from parted and sfdisk. All information
                   now stored as sectors NOT blocks.
V1.3 - 29 Mar 2021 tested with a blank drive and was crashing on the for loop around sfdisk,
                   if statement inserted to prevent.
V1.4 - 27 Oct 2021 blkadjust deleted and Tpart.psector populated after getting dLogical. Size converted
                   into blocks (was sectors)
V1.5 - 12 Nov 2021, pkexec added to lsblk command
V1.6 - 26 Nov 2021, populating Tpart.psector introduced an error, a for loop assuming # parts was > 0.
                    On a blank drive parts =0 resulting in index out of bounds, fixed by testing # of parts
                    beforehand.
V1.7 - 29 Dec 2021, complete rewrite.
-------------------------------------------------------------------------------}
procedure TDrive.GetDriveInfo;
begin
  //Device  already done, Selected is set by actions on checklistboxes, dBoot is determined in
  //readdrives, PartPrefix done when the TDrive is created in readdrives. PartCount populated when
  //TSystem is initialised in readdrives.

  //dModel from parted e.g. ATA WDC WDS240G2G0A
  dModel:= SysInfo.getModel(Device);
  //writeln(Device,' Model name=',dModel);
  //dLogical from parted drive logical sector size in bytes, duplicates sfdisk
  dLogical:= SysInfo.getLogSectorSize(Device);
  //writeln(Device,' logical sector size=',dlogical);
  //pTableType from parted msdos, gpt or unknown = no part table
  pTableType:= SysInfo.getPTableType(Device);
  //writeln(Device,' partition table=',pTableType);
  //dSerial the serial number of the drive
  dSerial:= SysInfo.getSerial(Device);
  //writeln(Device,' serial no=',dSerial,'xxxx');
  //dFirstPart start of the 1st partition in sectors, needed for grub on a legacy drive
  dFirstPart:= SysInfo.getFirstPartition(Device);
  //writeln(Device,' first sector start=',dFirstPart);
  //Size size of drive in blocks
  Size:= SysInfo.getDriveSize(Device,dLogical);
  //writeln(Device,' size=',Size,'blocks ',SizeStr(Size));
  //sfLabel from sfdisk this is actually label id which looks like a UUID for the drive
  //change to DriveUUID, not currently used
  DriveUUID:= SysInfo.getDriveUUID(Device);
  //writeln(Device,' drive UUID=',DriveUUID);
  //sfFirstLBA from sfdisk on a gpt drive would be 34
  if pTableType = 'gpt' then sfFirstLBA:= SysInfo.getFirstLBA(Device) else sfFirstLBA:= 0;
  //writeln(Device,' first LBA=',sfFirstLBA);
  //sfLastLBA from sfdisk on a gpt drive should be 34 sectors from the end of the drive
  if pTableType = 'gpt' then sfLastLBA:= SysInfo.getLastLBA(Device) else sfLastLBA:= 0;
  //writeln(Device,' last LBA=',sfLastLBA);
end; //end procedure TDrive.GetDriveInfo
{-------------------------------------------------------------------------------
FUNCTION TDRIVE.LISTSTR
Returns a drive string for display in checklist boxes.

The format is:

device : modelname : size : partition table type, e.g.
sda : abc123 : 240GB : Type MSDOS

Used by:

V1.0 - 24 Feb 2021
V1.1 - 07 Mar 2021 - changed as size is now in sectors not blocks
V1.2 - 27 Oct 2021 - blkadjust removed size should now be in 1k blocks
-------------------------------------------------------------------------------}
function TDrive.ListStr : string;
const
  delim = ' • '; //output to terminal would not show, but happy in listbox, copied from character map
                 //U+2022 BULLET
begin
  ListStr:= Device + delim;
  ListStr+= dModel + delim;
  ListStr+= SizeStr(Size)+ delim;
  ListStr+= rPartTableStr + ' ' + UpperCase(pTableType);
end; //end function TDrive.ListStr
{-------------------------------------------------------------------------------
FUNCTION TDRIVE.HINTSTR
Returns a hint string when hovering over a checklist box displaying drive information.

Used by:

V1.0 - 24 Feb 2021
V1.1 - 01 Mar 2021 - il8nstrings
V1.2 - 27 Oct 2021 - size now in 1k blocks, blkadjust removed
V1.3 - 05 Jan 2022 - size now reported in MB, not blocks and (GB)
-------------------------------------------------------------------------------}
function TDrive.HintStr : string;
const
  LF2 = LF + LF;
  tSep = ':  '; //separator from title to value
begin
  HintStr:= rMountptStr + tSep + device + LF2;
  HintStr+= rModelStr + tSep + dModel + LF2;
  HintStr+= rSerialStr + tSep + dSerial + LF2;
  HintStr+= rSizeStr + tSep + FormatMB(Size) + LF2;
  HintStr+= rPartTableStr + tSep + UpperCase(pTableType) + LF2;
  HintStr+= rPartitionStr + tSep + IntToStr(PartCount);
  HintStr+= LF + Space;
end; //end function TDrive.HintStr
{-------------------------------------------------------------------------------
FUNCTION TDRIVE.PRINT
Prints drive information to a file or terminal.

Not bothered with il8nstrings as this is for debug.

Used by:

V1.0 - 24 Feb 2021
V1.1 - 27 Oct 2021 - blkadjust removed, size is now in 1k blocks
V1.2 - 28 Oct 2021 - now directs output to a textfile and sector size added.
-------------------------------------------------------------------------------}
procedure TDrive.Print(var OutF : Textfile);
begin
  writeln(OutF,separator);
  if dBoot then writeln(OutF,'This is the boot drive');
  write(OutF,'Model = ',dModel);
  writeln(OutF,' : Serial no = ',dSerial);
  writeln(OutF,'Drive = ',device);
  writeln(OutF,'Partition label prefix = ',partprefix);
  writeln(OutF,'Sector size = ',dLogical,' bytes');
  writeln(OutF,'First partition starts at = ',inttostr(dFirstPart),' sectors');
  write(OutF,'Drive size = ',FormatFloat('#,##0',Size),' blocks (');
  writeln(OutF,SizeStr(Size),')');
  writeln(OutF,'Partition table = ',ptabletype);
end; //end procedure TDrive.print
{-------------------------------------------------------------------------------
FUNCTION TDRIVE.ISLOOP
Returns true if the drive has a loop partition table = there is no partition table, but the drive has
a valid filesystem on it starting at zero on the drive. It is what can happen if a newbie takes a
bare drive and formats it with disks (the 'create partition table' is hidden on the hamburger menu'.

Such a drive CANNOT be backed up by foxclone as there is no partition table, but it can be used as a
destination for a backup.

Used by:

V1.0 - 24 Feb 2021
-------------------------------------------------------------------------------}
function TDrive.isLoop : boolean; //true if boot drive
const
  LoopStr = 'loop';
begin
  isLoop:= ptabletype = LoopStr;
end; //end function TDrive.isLoop
{-------------------------------------------------------------------------------
FUNCTION TDRIVE.ISBLANK
Returns true if the drive has an unknown partition table type, most likely a blank or brand new
drive.

Used by:

V1.0 - 24 Feb 2021
-------------------------------------------------------------------------------}
function TDrive.isBlank : boolean; //true if boot drive
const
  UnknownStr = 'unknown';
begin
  isBlank:= ptabletype = UnknownStr;
end; //end function TDrive.isBlank
{-------------------------------------------------------------------------------
PROCEDURE TDRIVE.READSFDISK
Parses an sfdisk file into a TDrive (usually RDrive).

The elements populated are:
    pTableType  - msdos or gpt, sfdisk returns dos for msdos so this is changed.
    Device      - e.g. sda
    FirstLBA    - gpt only, normally = 34
    LastLBA     - gpt only, normally size of the drive less 34
    TotalParts  - from TParts.count
    PartPrefix  - by checking the partition number
    dFirstPart  - start of first partition in sectors, by checking start sector of each partition

It then creates a TPart and calls Partition.ParseSFdisk to parse the partition info in the sfdisk file.

ParseSFdisk populates the partition number as, e.g. sda1, the device name is then parsed out of pNumber.
pNumbers are then checked to see if there is a part prefix("p"), if present this is stripped out of
pnumber.

At the partition level, it also populates Part.Drive with Device + PartPrefix and TPart.BlockAdjust.

Parameters passed:
    sfFileName - the full pathname for the file created on a backup by sfdisk /dev/device --dump

Used by:

V1.0 - 24 Feb 2021
V1.1 - 15 Apr 2021, comments
V1.2 - 11 May 2021, FoxDialog replaced by FoxMsg
V1.3 - 28 Oct 2021, changes resulting from deleted blkadjust, now dumps a warning in the debug file
                    if unit is not sectors in sfdiskfile. Parameters to ParseSFdisk changed to include
                    sector size
V1.4 - 10 Dec 2021, checks on TotalParts added
-------------------------------------------------------------------------------}
procedure TDrive.ReadsfDisk(sfFileName : string);
const
  LabelStr = 'label:';
  DOSstr = 'dos';
  MSDOSstr = 'msdos';
  DeviceStr = 'device:';
  DevStr = '/dev/';
  Dev2Str = DeviceStr + ' ' + DevStr;
  UnitStr = 'unit:';
  SectorStr = 'sectors';
  FirstLBAstr = 'first-lba:';
  LastLBAstr = 'last-lba:';
var
  i : byte;
  sfData : TStringList;
  Numbers : set of char = ['0'..'9'];
begin
  DebugInfo(true,'TDrive.Readsfdisk',sfFileName);
  sfData:= TStringList.Create;
  sfData.LoadFromFile(sfFileName);
  for i:= 0 to sfData.Count - 1 do
      begin
        //get partition table type
        if pos(LabelStr,sfData[i]) = 1 then
           begin
             pTableType:= trim(replacestr(sfData[i],LabelStr,''));
             if pTableType = DOSstr then pTableType:= MSDOSstr; //consistency with parted
           end;
        //get the device, e.g. sda
        if pos(DeviceStr,sfData[i]) = 1 then Device:= trim(replacestr(sfData[i],Dev2Str,''));
        //is the unit sectors? if not dump an error into the debug file
        if pos(UnitStr,sfData[i]) = 1 then
           if pos(SectorStr,sfData[i]) = 0 then with self do
              DebugInfo(true,'Readsfdisk','WARNING sfdisk for ' + Device, 'model ' + Model,sfData[i]);
        //get first-lba, gpt only
        if pos(FirstLBAstr,sfData[i]) = 1 then FirstLBA:= StrToInt(trim(replacestr(sfData[i],FirstLBAstr,'')));
        //get last-lba, gpt only
        if pos(LastLBAstr,sfData[i]) = 1 then LastLBA:= StrToInt64(trim(replacestr(sfData[i],LastLBAstr,'')));
        //get the partition info
        if pos(DevStr,sfData[i]) = 1 then
           begin
             Partition:= TPart.create;
             Partition.ParseSFdisk(sfData[i],Self.dLogical);
             Parts.Add(Partition);
           end;
      end;
  //set TotalParts
  TotalParts:= Parts.Count;
  //at this point pnumber is of the form sda1 or mmcblk0p1, need to populate pDrive with either
  //sda or mmcblk0p, i.e. drive + partprefix and strip this from pNumber so that it is just '1'
  //first get rid of the device in pnumber
  //check on totalparts added
  if TotalParts > 0 then for i:= 0 to TotalParts - 1 do Parts[i].Number:= replacestr(Parts[i].Number,Device,'');
  //now find out if there is a part prefix, if there is strip it out of pnumber
  if not (Partitions[0].Number[1] in Numbers) then
     begin
       PartPrefix:= copy(Parts[0].Number,1,1);
       for i:= 0 to Parts.Count - 1 do Parts[i].Number:= replacestr(Parts[i].Number,PartPrefix,'');
     end;
  //populate pDrive with device + part prefix
  //check on total parts added
  if TotalParts > 0 then for i:= 0 to TotalParts - 1 do Parts[i].Drive:= Device + PartPrefix;
  //find the start of the first sector
  dFirstPart:= 99999;
  //check on totalparts added
  if TotalParts > 0 then for i:= 0 to TotalParts - 1 do
     if Parts[i].StartSector < dFirstPart then dFirstPart:= Parts[i].StartSector;
  sfData.Free;
end; //end procedure TDrive.ReadsfDisk
{-------------------------------------------------------------------------------
FUNCTION TDRIVE.CHECKPARTTABLE

Checks that the partition table has not changed since the backup was taken. It is called on the
drive being restored.

The function loads the sfdisk backup file into a TStringList and then dumps the partition table into
another TStringList and does a line-by-line comparison.

Any mismatch in either the length of the part tables or between individual lines results in
CheckPartTable returning false.

Note - don't need to escape any spaces in the filename, it is used in a pascal function not in a
terminal command.

Used by:

V1.0 - 19 Oct 2021
-------------------------------------------------------------------------------}
function TDrive.CheckPartTable(Fname : string) : boolean;
var
  BackPtable, CurrentPtable : TStringList;
  i : byte;
begin
  //load sfdisk file into one stringlist, load sfdisk dump into another compare line by line
  CheckPartTable:= true;
  BackPtable:= TStringList.Create;
  BackPtable.LoadFromFile(Fname);
  CurrentPtable:= TStringList.Create;
  getText('pkexec sfdisk /dev/' + Device + ' --dump',CurrentPtable);
  if BackPtable.Count = CurrentPtable.Count then
     begin
       for i:= 0 to BackPtable.Count - 1 do
           if BackPtable[i] <> CurrentPtable[i] then
              begin
                CheckPartTable:= false;
                break; //exit loop on first difference, may be more than one difference but just do first
              end;
     end
  else CheckPartTable:= false; //still want an error message
  BackPtable.Free;
  CurrentPtable.Free;
end; //end function TDrive.CheckPartTable
{--------------------------------------------------------------------------------------------------------
FUNCTION TDRIVE.GRUBBACKUP

With the introduction of hybrid boot in LM21 legacy install backup and restore of grub has become more
complicated. In V50 if the drive had a GPT partition table, the first sector on the drive and the gap
between the end of the partition table and 1st partition was not backed up.

Hybrid boot = GPT partition table, first 446 bytes of sector 0 are required and stage 2 lives in a
bios_grub partition.

Previously there was one function for grub, with hybrid boot this has been split up into separate
functions to try and simplify.

GrubBackup ALWAYS backs up the first 1MB (or wherever the first partition starts), irrespective of
whether the drive is GPT or msdos.

The function returns true if successful.

Parameters

    GrubFile - full pathname of the *.grub file

GrubBackup is called on the source drive.

When backing up, the dd copies the sectors from the start of the drive to the start of the first
partition (less one sector). This includes the partition table.

The function runs dd to backup grub:

    pkexec dd if=/dev/sda of=pathto/20211115-grub.bin count=xxx status=none


V1.0 - 25 Sep 2023
--------------------------------------------------------------------------------------------------------}
function TDrive.GrubBackup(GrubFile : string = '') : boolean;
const
  //default is a 512byte block size
  CmdStr = 'pkexec dd if=[source] of=[dest] status=none count=[countstr]';
  SourceStr = '[source]';
  DestinationStr = '[dest]';
  CountStr = '[countstr]';
  DeviceStr = '/dev/';
  BufSize = 1024;
var
  cmd, Buffer, grubOutput : string;
  cmdProcess : TProcess;
  BytesRead : integer;
begin
  DebugInfo(false,'TDrive.GrubBackup',GrubFile);
  GrubBackup:= true; //default is success
  //escape any spaces in the filename
  GrubFile:= replacestr(GrubFile,Space,EscSpace);
  //setup the grub command line
  cmd:= replacestr(CmdStr,DestinationStr,GrubFile);
  cmd:= replacestr(cmd,SourceStr,DeviceStr + Device);
  cmd:= replacestr(cmd,Countstr,IntToStr(dFirstPart-1));
  //run grub
  cmdProcess:= TProcess.Create(nil);
  grubOutput:= '';
  termCmd(cmd,cmdProcess);
  repeat
    setlength(Buffer,bufsize);
    BytesRead:= cmdProcess.stderr.Read(Buffer[1],length(Buffer));
    setlength(Buffer,BytesRead);
    grubOutput+= Buffer;
  until BytesRead = 0;
  cmdProcess.free;
  DebugInfo(false,'TDrive.GrubBackup','Output=',grubOutput);
  //check for errors
  if grubOutput <> '' then //error
     begin
       GrubBackup:= false;
       FoxMsg(rErrorStr,[rGrubErrorStr,cmd,grubOutput],mtError,[mbOK]);
     end;
end; //function TDrive.GrubBackup
{--------------------------------------------------------------------------------------------------------
FUNCTION TDRIVE.GRUBRESTORE

See GrubBackup for background.

GrubRestore is used by both restore and clone from file.

The function returns true if successful.

Parameters

    GrubFile - full pathname of the *.grub file

The function runs dd to restore grub.

GrubRestore is called on the target drive.

The function runs dd to restore grub

Restore: part 1
    pkexec dd if=pathto/20191107.grub of=/dev/sda bs=446 count=1 conv=notrunc status=none
Restore: part 2
    pkexec dd if=pathto/20191107.grub of=/dev/sda skip=1 seek=1 conv=notrunc status=none

Drive with a GPT partition table - do nothing
Drive with a GPT partition table AND a bios_grub partition - restore part 1.
Drive with an MSDOS partition table - restore parts 1 & 2.

Foxclone V50 does not save the grub file on a GPT drive, trying to restore it will cause an error.
Not restoring when a GPT drive has a bios_grub partiiton is a bug, nothing to be done about that with a
backup taken with V50.


V1.0 - 26 Sep 2023
--------------------------------------------------------------------------------------------------------}
function TDrive.GrubRestore(GrubFile : string = '') : boolean;
const
  //default is a 512byte block size
  CmdStr = 'pkexec dd if=[source] of=[dest] status=none';
  RestoreStr = ' bs=446 count=1 conv=notrunc'; //first part of restore
  RestoreStr2 = ' skip=1 seek=1 bs=512 conv=notrunc'; //second part of restore to skip the partition table
  SourceStr = '[source]';
  DestinationStr = '[dest]';
  DeviceStr = '/dev/';
  BufSize = 1024;
  sfdiskstr = '.sfdisk'; //these needed to get the sfdisk file
  grubstr = '.grub';
  gptpart = 'gpt'; //first line of sfdisk file
  bios_grubType = '21686148-6449-6E6F-744E-656564454649'; //as shown in sfdisk output

var
  cmd, Buffer, grubOutput : string;
  cmdProcess : TProcess;
  BytesRead : integer;
  //tIndex : shortint;
  sfList : TStringList; //content of the sfdisk file
  sfFileName : string;
  i : byte;
  gptFlag : boolean;
  bios_grubFlag : boolean;

begin
  DebugInfo(false,'TDrive.GrubRestore',GrubFile);
  GrubRestore:= true; //default is success
  sfFileName:= replacestr(GrubFile,grubstr,sfdiskstr); //get the sfdisk file name
  //note don't need to escape spaces using internal commands, e.g. loadfromfile
  GrubFile:= replacestr(GrubFile,Space,EscSpace); //escape any spaces in the filename
  //Is the backup from a gpt drive - first line of sfdisk file will tell us.
  //gpt drive = label: gpt
  //legacy drive = label: dos
  sfList:= TStringList.Create;
  sfList.LoadFromFile(sfFileName);
  //test for gpt partition table
  if pos(gptpart,sfList[0]) > 0 then gptFlag:= true else gptFlag:= false;
  //test for bios_grub partion
  bios_grubFlag:= false;
  for i:= 1 to sfList.count-1 do
      if pos(bios_grubType,sfList[i]) > 0 then
         begin
           bios_grubFlag:= true;
           break;
         end;
  sfList.Free;
  //GPT drive with no bios_grub partition must be booting UEFI, no need to do anything
  if gptFlag and not bios_grubFlag then exit;
  //Backup is legacy boot, either with a legacy partition table or GPT + bios_grub partition
  //Restore the first part = 1st 446 bytes of sector 0
  cmd:= replacestr(CmdStr,SourceStr,GrubFile) + RestoreStr;
  cmd:= replacestr(cmd,DestinationStr,DeviceStr + Device);
  //run grub
  cmdProcess:= TProcess.Create(nil);
  grubOutput:= '';
  termCmd(cmd,cmdProcess);
  repeat
    setlength(Buffer,bufsize);
    BytesRead:= cmdProcess.stderr.Read(Buffer[1],length(Buffer));
    setlength(Buffer,BytesRead);
    grubOutput+= Buffer;
  until BytesRead = 0;
  cmdProcess.free;
  DebugInfo(false,'TDrive.Grub','Output=',grubOutput);
  //check for errors
  if grubOutput <> '' then //error
     begin
       GrubRestore:= false;
       FoxMsg(rErrorStr,[rGrubErrorStr,cmd,grubOutput],mtError,[mbOK]);
     end;
  //nothing more to do if bios_grub partition
  if bios_grubFlag then exit;
  //what's left is legacy boot from a legacy partition table, need to restore the 2nd stage bootloader
  cmd:= replacestr(cmd,RestoreStr,RestoreStr2); //common to all
  grubOutput:= '';
  cmdProcess:= TProcess.Create(nil);
  termCmd(cmd,cmdProcess);
  repeat
    setlength(Buffer,bufsize);
    BytesRead:= cmdProcess.stderr.Read(Buffer[1],length(Buffer));
    setlength(Buffer,BytesRead);
    grubOutput+= Buffer;
  until BytesRead = 0;
  cmdProcess.free;
  //writeln('grub output stage2=',grubOutput);
  //check for errors
  DebugInfo(false,'TDrive.Grub','Output2=',grubOutput);
  if grubOutput <> '' then //error
     begin
       GrubRestore:= false;
       FoxMsg(rErrorStr,[rGrubErrorStr,cmd,grubOutput],mtError,[mbOK]);
     end;
end; //function TDrive.GrubRestore
{--------------------------------------------------------------------------------------------------------
FUNCTION TDRIVE.GRUBCLONE

See GrubBackup for background.

GrubClone is called on the source drive so that partition table type and presence of a bios_grub
partition (legacy boot) can be established for the clone.

EFI partition table no bios_grub partition - do nothing, grub is in the EFI partition.

EFI partition table AND bios_grub partition - clone the first 446 bytes of the drive.

MSDOS partition table - clone 1st 446 bytes & 2nd stage between end of part table & start 1st partition.

Clone: part 1
    pkexec dd if=/dev/source of=/dev/target bs=446 count=1 conv=notrunc status=none
Clone: part 2
    pkexec dd if=/dev/source of=/dev/target skip=1 seek=1 conv=notrunc status=none

V1.0 - 29 Sep 2023
V1.1 - 04 Oct 2023 on legacy 2nd stage needs to be limited otherwise does whole drive. need count= and
                   work out count from where first part starts.
--------------------------------------------------------------------------------------------------------}
function TDrive.GrubClone : boolean;
const
  //default is a 512byte block size
  CmdStr = 'pkexec dd if=[source] of=[dest] status=none';
  RestoreStr = ' bs=446 count=1 conv=notrunc'; //first part of restore
  RestoreStr2 = ' skip=1 seek=1 bs=512 count=[blocks] conv=notrunc'; //second part of restore to skip the partition table
  SourceStr = '[source]';
  DestinationStr = '[dest]';
  DeviceStr = '/dev/';
  BufSize = 1024;
  bios_grubstr = 'bios_grub';
  BlockStr = '[blocks]';
var
  cmd, Buffer, grubOutput : string;
  cmdProcess : TProcess;
  BytesRead : integer;
  tIndex : shortint;
  GPTflag : boolean;
  bios_grubFlag : boolean;
  i : byte;
  Blocks : integer;
begin
  //writeln('grub clone called');
  DebugInfo(false,'TDrive.GrubClone',Device);
  GrubClone:= true; //default is success
  //check if GPT or legacy parttable
  if pTableType = 'gpt' then GPTflag:= true else GPTflag:= false;
  //find if there is a bios_grub partition
  bios_grubFlag:= false;
  for i:= 0 to PartCount - 1 do if pos(bios_grubstr,Parts[i].Pflags) > 0 then bios_grubFlag:= true;
  //with a GPT part table and no bios_grub partition nothing to do
  if GPTflag and not bios_grubFlag then exit;
  //find the target drive (sfdisk is called on the source for a clone)
  tIndex:= MyDrives.getDrive(acTarget);
  //Clone is legacy boot, either with a legacy partition table or GPT + bios_grub partition
  //Restore the first part = 1st 446 bytes of sector 0
  cmd:= replacestr(CmdStr,SourceStr,DeviceStr + Device) + RestoreStr;
  cmd:= replacestr(cmd,DestinationStr,DeviceStr + MyDrives[tIndex].Device);
  //run grub
  writeln('grub first stage=',cmd);
  cmdProcess:= TProcess.Create(nil);
  grubOutput:= '';
  termCmd(cmd,cmdProcess);
  repeat
    setlength(Buffer,bufsize);
    BytesRead:= cmdProcess.stderr.Read(Buffer[1],length(Buffer));
    setlength(Buffer,BytesRead);
    grubOutput+= Buffer;
  until BytesRead = 0;
  cmdProcess.free;
  DebugInfo(false,'TDrive.GrubClone','Output=',grubOutput);
  //check for errors
  if grubOutput <> '' then //error
     begin
       GrubClone:= false;
       FoxMsg(rErrorStr,[rGrubErrorStr,cmd,grubOutput],mtError,[mbOK]);
     end;
  //nothing more to do if bios_grub partition
  if bios_grubFlag then exit;
  //what's left is legacy boot from a legacy partition table, need to clone the 2nd stage bootloader
  //what is number of blocks to copy?
  Blocks:= MyDrives[tIndex].dFirstPart - 1;
  cmd:= replacestr(cmd,RestoreStr,RestoreStr2);
  cmd:= replacestr(cmd,BlockStr,IntToStr(Blocks));
  writeln('grub second stage=',cmd);
  grubOutput:= '';
  cmdProcess:= TProcess.Create(nil);
  termCmd(cmd,cmdProcess);
  repeat
    setlength(Buffer,bufsize);
    BytesRead:= cmdProcess.stderr.Read(Buffer[1],length(Buffer));
    setlength(Buffer,BytesRead);
    grubOutput+= Buffer;
  until BytesRead = 0;
  cmdProcess.free;
  //check for errors
  DebugInfo(false,'TDrive.GrubClone','Output2=',grubOutput);
  if grubOutput <> '' then //error
     begin
       GrubClone:= false;
       FoxMsg(rErrorStr,[rGrubErrorStr,cmd,grubOutput],mtError,[mbOK]);
     end;
end; //function TDrive.GrubClone
{-------------------------------------------------------------------------------
FUNCTION TDRIVE.SFDISK

Backs up, restores and clones the partition table. The action taken is dependent on Action.

It returns true if the action was successful, false otherwise. On error a message dialog is displayed.

Parameters
    Action - where this is acbackup, acRestore, acClone or acCloneF.
    SfDevice - a full pathname, e.g. the backup filename

What happens is dependent on Action:

    acBackup  : sfdisk is called on the source drive and sfFile is the filename to save the partition
                table to. sfdisk is run and the output saved to sfFile
    acRestore : sfdisk is called on the destination drive and sfFile is the filename containing the
                sfdisk backup. sfFile is passed to sfdisk to restore the part table.
    acClone   : sfdisk is called on the source drive. The destination drive is found (set acTarget).
                A dump of the partition table for the source drive is created as a temp file. This is
                then passed to sfdisk to restore to the destination drive.
    acCloneF  : sfdisk is called on the destination drive and sfFile is the filename containing the
                sfdisk backup.

With both acCloneF and acClone, the part table from the source is checked for last lba and adjusted to
match the size of the new drive. The last lba is only present for a gpt format drive, it is there to
provide space for the backup partition table, this occupies the last 34 sectors on the drive.
The part table is saved as a temp file and this is used to restore to the new drive.

Backup: sfdisk -d /dev/sda > sda.out to a text file
Restore: sfdisk /dev/sdf < sda.out   from a text file

on a backup sfdisk produces no output to stderr on success - check in terminal - none
now check with an error
pkexec sfdisk -d /dev/sdX > /run/timeshift/backup/Testing\ foxclone50/20211020.sfdisk
generates output

Used by:

V1.0 - 20 Oct 2021
V1.1 - 14 Nov 2021, restore added
V1.2 - 16 Nov 2021, --force removed, do not want it overwriting the part table if there is an error
V1.3 - 25 Nov 2021, on acCloneF has to check the sfdisk file and adjust the last lba if present to
                    match the new drive.
V1.4 - 27 Nov 2021, amended for acClone.
V1.5 - 04 Jan 2022, pascal statements e.g. tstringlist.loadfromfile(filename) do NOT want spaces escaped.
                    The sf filename is escaped early on and then loaded into a tstringlist in clone from
                    file = error. Fixed.
V1.6 - 28 Jan 2022, was checking to see if gparted was running. If it was the prog window is left
                    with none of the buttons enabled and no simple way to close the form. Removed and
                    the check is now in RunFoxclone in the partclone unit.
V1.7 - 26 Oct 2023, problem with sfdisk, later versions insert a sector-size line into the dump file. If
                    foxclone is then run with an earlier version of sfdisk, e.g. backup taken running
                    under jammy and restore done under bionic, it will crash as the earlier version of
                    sfdisk does not understand sector-size. Function changed to check for and remove a
                    sector-size line in the dump file so that it will work with all versions of foxclone
                    irrespective of what was used to take the backup in the first place.
-------------------------------------------------------------------------------}
function TDrive.sfdisk(Action : TFoxType; sfFile : string = '') : boolean;
const
  //default is a 512byte block size
  BackStr = 'pkexec sfdisk -d [source] > [dest] 2>&1';
  RestoreStr = 'pkexec sfdisk /dev/[dest] < [source]';
  SourceStr = '[source]';
  DestinationStr = '[dest]';
  DeviceStr = '/dev/';
  LastLBAstr = 'last-lba: ';
  BufSize = 4096; //can generate a lot of output
  TempsfFile = DefaultFolder + 'temp.sfdisk';
  SectorStr = 'sector-size:'; //addition to dump file in later versions of sfdisk
var
  cmd, Buffer, sfOutput : string;
  cmdProcess : TProcess;
  BytesRead : integer;
  sfList : TStringList;
  i, tIndex, LineRef : shortint; //tindex is the index for the target drive
begin
  sfdisk:= true; //default is success
  DebugInfo(false,'TDrive.sfdisk','sfFile',sfFile);
  //escape any spaces in the filename
  sfFile:= replacestr(sfFile,Space,EscSpace);
  //setup the sfdisk command line
  case Action of
       acbackup  : begin //sfFile is the destination file
                     cmd:= replacestr(BackStr,DestinationStr,sfFile);
                     cmd:= replacestr(cmd,SourceStr,DeviceStr + Device);
                   end;
       acRestore : begin //sfFile is the source file
                     cmd:= replacestr(RestoreStr,DestinationStr,Device);
                     cmd:= replacestr(cmd,SourceStr,sfFile);
                   end;
       acCloneF  : begin //need to amend the last lba line in the file if cloning from a gpt drive backup
                     cmd:= replacestr(RestoreStr,DestinationStr,Device);
                     cmd:= replacestr(cmd,SourceStr,sfFile); //default no change to sf file
                     //load the sfdisk file
                     sfList:= TStringList.Create;
                     //pascal statements do NOT handle escaped spaces so replace
                     sfList.LoadFromFile(replacestr(sfFile,ESCspace,Space));
                     //now find the lastlba line
                     for i:= 0 to sfList.Count - 1 do if pos(LastLBAstr,sfList[i]) = 1 then
                         begin
                           sfList[i]:= LastLBAstr + IntToStr(BlockToSector(Size,dLogical) - 34);
                           //writeln('drive size=',BlockToSector(Size,dLogical),' last lba=',sfList[i]);
                           //save the file
                           sfList.SaveToFile(TempsfFile);
                           cmd:= replacestr(cmd,sfFile,TempsfFile); //change sf file
                           break;
                         end;
                     sfList.free;
                   end;
       acClone   : begin //need to create the sfdisk file and then amend for last lba on target
                     //find the target drive (sfdisk is called on the source for a clone)
                     tIndex:= MyDrives.getDrive(acTarget);
                     if tIndex = -1 then SystemError(24,'TDrive.sfdisk');
                     cmd:= replacestr(RestoreStr,DestinationStr,MyDrives[tIndex].Device);
                      //get part table for the source drive
                     sfList:= TStringList.Create;
                     getText('pkexec sfdisk --dump /dev/' + Device,sfList);
                     //amend for last lba if present
                     for i:= 0 to sfList.Count - 1 do if pos(LastLBAstr,sfList[i]) = 1 then
                         begin
                           with MyDrives[tIndex] do sfList[i]:= LastLBAstr + IntToStr(BlockToSector(Size,dLogical) - 34);
                           //writeln('last lba on destination=',sfList[i]);
                           //save the file
                           break;
                         end;
                     sfList.SaveToFile(TempsfFile);
                     cmd:= replacestr(cmd,SourceStr,TempsfFile); //change sf file
                     sfList.free;
                   end;
  end; //case
  writeln('TDrive.sfdisk, sfdisk cmd=',cmd);
  //run sfdisk
  cmdProcess:= TProcess.Create(nil);
  sfOutput:= '';
  termCmd(cmd,cmdProcess);
  repeat
    setlength(Buffer,bufsize);
    BytesRead:= cmdProcess.output.Read(Buffer[1],length(Buffer));
    setlength(Buffer,BytesRead);
    sfOutput+= Buffer;
  until BytesRead = 0;
  //writeln('sfdisk output=',sfOutput);
  //check for errors - need to check if it writes to stderr or stdout
  //from testing, "failed" is what needs to be checked, no "error" in output
  //sfdisk always generates output on restore
  DebugInfo(false,'TDrive.sfdisk','output=',sfOutput);
  //if a backup, check for sector-size line in the dump file and remove it
  //this to stop a restore crashing if run with an earlier version of sfdisk that does not understand
  //sector-size.
  if Action = acbackup then
     begin
       LineRef:= -1;
       sfList:= TStringList.Create;
       //removed any escape chars = \ in filename
       sfFile:= replacestr(sfFile,'\','');
       writeln('TDrive.sfdisk, sf filename=',sfFile);
       sfList.LoadFromFile(sfFile);
       writeln('TDrive.sfdisk, after load from file');
       for i:= 0 to sfList.Count - 1 do
           if pos(SectorStr,sfList[i]) = 1 then //found
              begin
                LineRef:= i;
                break;
              end;
       if LineRef <> -1 then
          begin
            sfList.Delete(LineRef);
            sfList.SaveToFile(sfFile);
          end;
       sfList.free;
     end;
  if (pos('failed',sfOutput) > 0) or (pos('FAILED',sfOutput) > 0) then //error
     begin
       FoxMsg(rErrorStr,[rSfdiskErrorStr,cmd,sfOutput],mtError,[mbOK]);
       sfdisk:= false;
     end;
  cmdProcess.free;
end; //function TDrive.sfdisk
{-------------------------------------------------------------------------------
PROCEDURE TSYSTEM.SETFAT32SPLIT
This procedure checks the destination partition on a backup and if the filesystem is fat32 turns on
split so that image files are split. fat32 is max file size of 4GB.

The procedure does nothing is the destination is not fat32.

parameters
    OnOff - boolean, when true sets split if it had not been set on by the user. When false, checks to
    see if the user had set split on and if not turns it off.

Much thought about where to do this procedure. On clicking on the part in box4 - how do you then turn it
off if the user does more than one thing or then chooses a different partition? As a procedure in
partitions, e.g. in BackStr (building the partclone command line)  - partitions has no knowledge of
drives and the calling procedure would have to find the destination first.

Thus implemented as a method of mydrives at system level. It is called twice by the runbackup
procedure (not needed anywhere else). Once to turn split on, and on conclusion to turn split off (if the
user had not set it).

Used by:

V1.0 - 23 Oct 2021
V1.1 - 10 Dec 2021, checks on totalparts added
-------------------------------------------------------------------------------}
procedure TSystem.SetFat32Split(OnOff : boolean);
const
  fat32Str = 'fat32';
var
  i,dIndex,pIndex : shortint;
begin
  dIndex:= Self.getDrive(acTarget); //target drive for backup
  if dIndex = -1 then SystemError(25,'TSystem.SetFat32Split');
  //find destination partition
  pIndex:= -1;
  if Self[dIndex].TotalParts > 0 then for i:= 0 to Self[dIndex].TotalParts - 1 do
      if Self[dIndex].Partitions[i].selected then pIndex:= i;
  if pIndex = -1 then SystemError(26,'TSystem.SetFat32Split');
  //check fs
  if Self[dIndex].Partitions[pIndex].FileSystem <> fat32Str then exit; //nothing to do
  //is fat32 turn on/off split
  //if OnOff then writeln('fat32 destination, turning split on') else writeln('fat32 destination, turning split OFF');
  if OnOff then DebugInfo(true,'TSystem.SetFat32Split', 'ON fat32 dest') else DebugInfo(true,'TSystem.SetFat32Split', 'OFF');
  if OnOff then with Defaults do //check and set split
     case BackSettings of
          seNone : BackSettings:= seSplit;
          seComp : BackSettings:= seBoth;
     end
  else
     if not Defaults.IsSplit then with Defaults do//user did not set split on, so need to reset
        case BackSettings of
             seSplit : BackSettings:= seNone;
             seBoth  : BackSettings:= seComp;
        end;
end;
{-------------------------------------------------------------------------------
FUNCTION TDRIVE.PARTPROBE
Re-reads the partition table after changes. In previous versions of foxclone partprobe was used.
Problem was I could not get it to generate any output so not able to test for errors.

Did try blockdev, sudo blockdev --rereadpt /dev/sda, but reverted to partprobe.

Partprobe does generate output on an error.

Function returns true if okay and false on error.

As of 30 Dec 2021 not being used.

Used by:

V1.0 - 14 Oct 2021 blockdev version
V1.1 - 16 Nov 2021 partprobe version
-------------------------------------------------------------------------------}
function TDrive.PartProbe : boolean;
const
  bufsize = 128;
  Template = 'pkexec partprobe /dev/[device]';
  DeviceStr = '[device]';
var
  cmdStr, Buffer, probeOutput : string;
  cmdProcess : TProcess;
  BytesRead : integer;
begin
  PartProbe:= true;
  probeOutput:= '';
  cmdStr:= replacestr(Template,DeviceStr,Device);
  //writeln('partprobe cmd =',cmdStr);
  cmdProcess:= TProcess.Create(nil);
  termCmd(cmdStr,cmdProcess);
  repeat
    setlength(Buffer,bufsize);
    BytesRead:= cmdProcess.stderr.Read(Buffer[1],length(Buffer));
    setlength(Buffer,BytesRead);
    probeOutput+= Buffer;
  until BytesRead = 0;
  if probeOutput <> '' then
     begin
       PartProbe:= false;
       FoxMsg(rErrorStr,[rPtableErrorStr + Device,probeOutput],mtError,[mbOK]);
     end;
  cmdProcess.free;
end; //end function TDrive.PartProbe

end.

